@@ -2,8 +2,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-static const char fileIdent[] = "$Id$";
-
 /* initialize an fmtext with no data */
 void owl_fmtext_init_null(owl_fmtext *f)
 {
@@ -27,7 +25,7 @@ void owl_fmtext_clear(owl_fmtext *f)
   f->default_bgcolor = OWL_COLOR_DEFAULT;
 }
 
-void _owl_fmtext_realloc(owl_fmtext *f, int newlen) /*noproto*/
+static void _owl_fmtext_realloc(owl_fmtext *f, int newlen)
 {
     if(newlen + 1 > f->bufflen) {
       f->textbuff = owl_realloc(f->textbuff, newlen + 1);
@@ -44,7 +42,7 @@ int owl_fmtext_is_format_char(gunichar c)
 /* append text to the end of 'f' with attribute 'attr' and color
  * 'color'
  */
-void owl_fmtext_append_attr(owl_fmtext *f, char *text, char attr, short fgcolor, short bgcolor)
+void owl_fmtext_append_attr(owl_fmtext *f, const char *text, char attr, short fgcolor, short bgcolor)
 {
   char attrbuff[6];
   int newlen, a = 0, fg = 0, bg = 0;
@@ -58,21 +56,15 @@ void owl_fmtext_append_attr(owl_fmtext *f, char *text, char attr, short fgcolor,
   _owl_fmtext_realloc(f, newlen);
 
   /* Set attributes */
-  if (a) {
-    memset(attrbuff,0,6);
-    g_unichar_to_utf8(OWL_FMTEXT_UC_ATTR | attr, attrbuff);
-    strcat(f->textbuff, attrbuff);      
-  }
-  if (fg) {
-    memset(attrbuff,0,6);
-    g_unichar_to_utf8(OWL_FMTEXT_UC_FGCOLOR | fgcolor, attrbuff);
-    strcat(f->textbuff, attrbuff);      
-  }
-  if (bg) {
-    memset(attrbuff,0,6);
-    g_unichar_to_utf8(OWL_FMTEXT_UC_BGCOLOR | bgcolor, attrbuff);
-    strcat(f->textbuff, attrbuff);      
-  }
+  if (a)
+    strncat(f->textbuff, attrbuff,
+	    g_unichar_to_utf8(OWL_FMTEXT_UC_ATTR | attr, attrbuff));
+  if (fg)
+    strncat(f->textbuff, attrbuff,
+	    g_unichar_to_utf8(OWL_FMTEXT_UC_FGCOLOR | fgcolor, attrbuff));
+  if (bg)
+    strncat(f->textbuff, attrbuff,
+	    g_unichar_to_utf8(OWL_FMTEXT_UC_BGCOLOR | bgcolor, attrbuff));
   
   strcat(f->textbuff, text);
 
@@ -84,13 +76,13 @@ void owl_fmtext_append_attr(owl_fmtext *f, char *text, char attr, short fgcolor,
 }
 
 /* Append normal, uncolored text 'text' to 'f' */
-void owl_fmtext_append_normal(owl_fmtext *f, char *text)
+void owl_fmtext_append_normal(owl_fmtext *f, const char *text)
 {
   owl_fmtext_append_attr(f, text, OWL_FMTEXT_ATTR_NONE, OWL_COLOR_DEFAULT, OWL_COLOR_DEFAULT);
 }
 
 /* Append normal, uncolored text specified by format string to 'f' */
-void owl_fmtext_appendf_normal(owl_fmtext *f, char *fmt, ...)
+void owl_fmtext_appendf_normal(owl_fmtext *f, const char *fmt, ...)
 {
   va_list ap;
   char *buff;
@@ -101,28 +93,29 @@ void owl_fmtext_appendf_normal(owl_fmtext *f, char *fmt, ...)
   if (!buff)
     return;
   owl_fmtext_append_attr(f, buff, OWL_FMTEXT_ATTR_NONE, OWL_COLOR_DEFAULT, OWL_COLOR_DEFAULT);
+  owl_free(buff);
 }
 
 /* Append normal text 'text' to 'f' with color 'color' */
-void owl_fmtext_append_normal_color(owl_fmtext *f, char *text, int fgcolor, int bgcolor)
+void owl_fmtext_append_normal_color(owl_fmtext *f, const char *text, int fgcolor, int bgcolor)
 {
   owl_fmtext_append_attr(f, text, OWL_FMTEXT_ATTR_NONE, fgcolor, bgcolor);
 }
 
 /* Append bold text 'text' to 'f' */
-void owl_fmtext_append_bold(owl_fmtext *f, char *text)
+void owl_fmtext_append_bold(owl_fmtext *f, const char *text)
 {
   owl_fmtext_append_attr(f, text, OWL_FMTEXT_ATTR_BOLD, OWL_COLOR_DEFAULT, OWL_COLOR_DEFAULT);
 }
 
 /* Append reverse video text 'text' to 'f' */
-void owl_fmtext_append_reverse(owl_fmtext *f, char *text)
+void owl_fmtext_append_reverse(owl_fmtext *f, const char *text)
 {
   owl_fmtext_append_attr(f, text, OWL_FMTEXT_ATTR_REVERSE, OWL_COLOR_DEFAULT, OWL_COLOR_DEFAULT);
 }
 
 /* Append reversed and bold, uncolored text 'text' to 'f' */
-void owl_fmtext_append_reversebold(owl_fmtext *f, char *text)
+void owl_fmtext_append_reversebold(owl_fmtext *f, const char *text)
 {
   owl_fmtext_append_attr(f, text, OWL_FMTEXT_ATTR_REVERSE | OWL_FMTEXT_ATTR_BOLD, OWL_COLOR_DEFAULT, OWL_COLOR_DEFAULT);
 }
@@ -151,7 +144,7 @@ void owl_fmtext_colorizebg(owl_fmtext *f, int color)
 }
 
 /* Internal function. Parse attrbute character. */
-void _owl_fmtext_update_attributes(gunichar c, char *attr, short *fgcolor, short *bgcolor) /*noproto*/
+static void _owl_fmtext_update_attributes(gunichar c, char *attr, short *fgcolor, short *bgcolor)
 {
   if ((c & OWL_FMTEXT_UC_ATTR) == OWL_FMTEXT_UC_ATTR) {
     *attr = c & OWL_FMTEXT_UC_ATTR_MASK;
@@ -171,9 +164,9 @@ void _owl_fmtext_update_attributes(gunichar c, char *attr, short *fgcolor, short
 }
 
 /* Internal function. Scan for attribute characters. */
-void _owl_fmtext_scan_attributes(owl_fmtext *f, int start, char *attr, short *fgcolor, short *bgcolor) /*noproto*/
+static void _owl_fmtext_scan_attributes(const owl_fmtext *f, int start, char *attr, short *fgcolor, short *bgcolor)
 {
-  char *p;
+  const char *p;
   p = strchr(f->textbuff, OWL_FMTEXT_UC_STARTBYTE_UTF8);
   while (p && p < f->textbuff + start) {
     _owl_fmtext_update_attributes(g_utf8_get_char(p), attr, fgcolor, bgcolor);
@@ -181,11 +174,11 @@ void _owl_fmtext_scan_attributes(owl_fmtext *f, int start, char *attr, short *fg
   }
 }  
 
-/* Internal function.  Append text from 'in' between index 'start' and
- * 'stop', inclusive, to the end of 'f'. This function works with
- * bytes.
+/* Internal function.  Append text from 'in' between index 'start'
+ * inclusive and 'stop' exclusive, to the end of 'f'. This function
+ * works with bytes.
  */
-void _owl_fmtext_append_fmtext(owl_fmtext *f, owl_fmtext *in, int start, int stop) /*noproto*/
+static void _owl_fmtext_append_fmtext(owl_fmtext *f, const owl_fmtext *in, int start, int stop)
 {
   char attrbuff[6];
   int newlen, a = 0, fg = 0, bg = 0;
@@ -200,26 +193,20 @@ void _owl_fmtext_append_fmtext(owl_fmtext *f, owl_fmtext *in, int start, int sto
 
   /* We will reset to defaults after appending the text. We may need
      to set initial attributes. */
-  newlen=strlen(f->textbuff)+(stop-start+1) + (4 * (a + fg + bg)) + 12;
+  newlen=strlen(f->textbuff)+(stop-start) + (4 * (a + fg + bg)) + 12;
   _owl_fmtext_realloc(f, newlen);
 
-  if (a) {
-    memset(attrbuff,0,6);
-    g_unichar_to_utf8(OWL_FMTEXT_UC_ATTR | attr, attrbuff);
-    strcat(f->textbuff, attrbuff);      
-  }
-  if (fg) {
-    memset(attrbuff,0,6);
-    g_unichar_to_utf8(OWL_FMTEXT_UC_FGCOLOR | fgcolor, attrbuff);
-    strcat(f->textbuff, attrbuff);      
-  }
-  if (bg) {
-    memset(attrbuff,0,6);
-    g_unichar_to_utf8(OWL_FMTEXT_UC_BGCOLOR | bgcolor, attrbuff);
-    strcat(f->textbuff, attrbuff);      
-  }
+  if (a)
+    strncat(f->textbuff, attrbuff,
+	    g_unichar_to_utf8(OWL_FMTEXT_UC_ATTR | attr, attrbuff));
+  if (fg)
+    strncat(f->textbuff, attrbuff,
+	    g_unichar_to_utf8(OWL_FMTEXT_UC_FGCOLOR | fgcolor, attrbuff));
+  if (bg)
+    strncat(f->textbuff, attrbuff,
+	    g_unichar_to_utf8(OWL_FMTEXT_UC_BGCOLOR | bgcolor, attrbuff));
 
-  strncat(f->textbuff, in->textbuff+start, stop-start+1);
+  strncat(f->textbuff, in->textbuff+start, stop-start);
 
   /* Reset attributes */
   strcat(f->textbuff, OWL_FMTEXT_UTF8_BGDEFAULT);
@@ -231,7 +218,7 @@ void _owl_fmtext_append_fmtext(owl_fmtext *f, owl_fmtext *in, int start, int sto
 }
 
 /* append fmtext 'in' to 'f' */
-void owl_fmtext_append_fmtext(owl_fmtext *f, owl_fmtext *in)
+void owl_fmtext_append_fmtext(owl_fmtext *f, const owl_fmtext *in)
 {
   _owl_fmtext_append_fmtext(f, in, 0, in->textlen);
 
@@ -249,12 +236,12 @@ void owl_fmtext_append_spaces(owl_fmtext *f, int nspaces)
 /* Return a plain version of the fmtext.  Caller is responsible for
  * freeing the return
  */
-char *owl_fmtext_print_plain(owl_fmtext *f)
+char *owl_fmtext_print_plain(const owl_fmtext *f)
 {
   return owl_strip_format_chars(f->textbuff);
 }
 
-void _owl_fmtext_wattrset(WINDOW *w, int attrs) /*noproto*/
+static void _owl_fmtext_wattrset(WINDOW *w, int attrs)
 {
   wattrset(w, A_NORMAL);
   if (attrs & OWL_FMTEXT_ATTR_BOLD) wattron(w, A_BOLD);
@@ -262,43 +249,37 @@ void _owl_fmtext_wattrset(WINDOW *w, int attrs) /*noproto*/
   if (attrs & OWL_FMTEXT_ATTR_UNDERLINE) wattron(w, A_UNDERLINE);
 }
 
-void _owl_fmtext_update_colorpair(short fg, short bg, short *pair) /*noproto*/
+static void _owl_fmtext_update_colorpair(short fg, short bg, short *pair)
 {
   if (owl_global_get_hascolors(&g)) {
     *pair = owl_fmtext_get_colorpair(fg, bg);
   }
 }
 
-void _owl_fmtext_wcolor_set(WINDOW *w, short pair) /*noproto*/
+static void _owl_fmtext_wcolor_set(WINDOW *w, short pair)
 {
   if (owl_global_get_hascolors(&g)) {
       wcolor_set(w,pair,NULL);
+      wbkgdset(w, COLOR_PAIR(pair));
   }
 }
 
 /* add the formatted text to the curses window 'w'.  The window 'w'
  * must already be initiatlized with curses
  */
-void _owl_fmtext_curs_waddstr(owl_fmtext *f, WINDOW *w, int do_search) /*noproto*/
+static void _owl_fmtext_curs_waddstr(const owl_fmtext *f, WINDOW *w, int do_search)
 {
   /* char *tmpbuff; */
   /* int position, trans1, trans2, trans3, len, lastsame; */
   char *s, *p;
   char attr;
-  short fg, bg, pair;
-  int search_results, search_len;
+  short fg, bg, pair = 0;
   
   if (w==NULL) {
     owl_function_debugmsg("Hit a null window in owl_fmtext_curs_waddstr.");
     return;
   }
 
-  search_results = (do_search
-		    ? owl_fmtext_search(f, owl_global_get_search_string(&g))
-		    : 0);
-  search_len = (search_results
-		? strlen(owl_global_get_search_string(&g))
-		: 0);
   s = f->textbuff;
   /* Set default attributes. */
   attr = f->default_attrs;
@@ -317,31 +298,34 @@ void _owl_fmtext_curs_waddstr(owl_fmtext *f, WINDOW *w, int do_search) /*noproto
    
       tmp = p[0];
       p[0] = '\0';
-      if (search_results) {
+      if (do_search && owl_global_is_search_active(&g)) {
 	/* Search is active, so highlight search results. */
-	char tmp2, *ss;
-	ss = stristr(s, owl_global_get_search_string(&g));
-	while (ss) {
+	char tmp2;
+	int start, end;
+	while (owl_regex_compare(owl_global_get_search_re(&g), s, &start, &end) == 0) {
+	  /* Prevent an infinite loop matching the empty string. */
+	  if (end == 0)
+	    break;
+
 	  /* Found search string, highlight it. */
 
-	  tmp2 = ss[0];
-	  ss[0] = '\0';
+	  tmp2 = s[start];
+	  s[start] = '\0';
 	  waddstr(w, s);
-	  ss[0] = tmp2;
+	  s[start] = tmp2;
 
 	  _owl_fmtext_wattrset(w, attr ^ OWL_FMTEXT_ATTR_REVERSE);
 	  _owl_fmtext_wcolor_set(w, pair);
 	  
-	  tmp2 = ss[search_len];
-	  ss[search_len] = '\0';
-	  waddstr(w, ss);
-	  ss[search_len] = tmp2;
+	  tmp2 = s[end];
+	  s[end] = '\0';
+	  waddstr(w, s + start);
+	  s[end] = tmp2;
 
 	  _owl_fmtext_wattrset(w, attr);
 	  _owl_fmtext_wcolor_set(w, pair);
 
-	  s = ss + search_len;
-	  ss = stristr(s, owl_global_get_search_string(&g));
+	  s += end;
 	}
       }
       /* Deal with remaining part of string. */
@@ -353,7 +337,7 @@ void _owl_fmtext_curs_waddstr(owl_fmtext *f, WINDOW *w, int do_search) /*noproto
       attr = f->default_attrs;
       fg = f->default_fgcolor;
       bg = f->default_bgcolor;
-      while (p && owl_fmtext_is_format_char(g_utf8_get_char(p))) {
+      while (owl_fmtext_is_format_char(g_utf8_get_char(p))) {
 	_owl_fmtext_update_attributes(g_utf8_get_char(p), &attr, &fg, &bg);
 	p = g_utf8_next_char(p);
       }
@@ -374,24 +358,62 @@ void _owl_fmtext_curs_waddstr(owl_fmtext *f, WINDOW *w, int do_search) /*noproto
   if (s) {
     waddstr(w, s);
   }
+  wbkgdset(w, 0);
 }
 
-void owl_fmtext_curs_waddstr(owl_fmtext *f, WINDOW *w)
+void owl_fmtext_curs_waddstr(const owl_fmtext *f, WINDOW *w)
 {
-  _owl_fmtext_curs_waddstr(f, w, owl_global_is_search_active(&g));
+  _owl_fmtext_curs_waddstr(f, w, 1);
 }
 
-void owl_fmtext_curs_waddstr_without_search(owl_fmtext *f, WINDOW *w)
+void owl_fmtext_curs_waddstr_without_search(const owl_fmtext *f, WINDOW *w)
 {
   _owl_fmtext_curs_waddstr(f, w, 0);
 }
 
+/* Expands tabs. Tabs are expanded as if given an initial indent of start. */
+void owl_fmtext_expand_tabs(const owl_fmtext *in, owl_fmtext *out, int start) {
+  int col = start, numcopied = 0;
+  char *ptr;
+
+  /* Copy the default attributes. */
+  out->default_attrs = in->default_attrs;
+  out->default_fgcolor = in->default_fgcolor;
+  out->default_bgcolor = in->default_bgcolor;
+
+  for (ptr = in->textbuff;
+       ptr < in->textbuff + in->textlen;
+       ptr = g_utf8_next_char(ptr)) {
+    gunichar c = g_utf8_get_char(ptr);
+    int chwidth;
+    if (c == '\t') {
+      /* Copy up to this tab */
+      _owl_fmtext_append_fmtext(out, in, numcopied, ptr - in->textbuff);
+      /* and then copy spaces for the tab. */
+      chwidth = OWL_TAB_WIDTH - (col % OWL_TAB_WIDTH);
+      owl_fmtext_append_spaces(out, chwidth);
+      col += chwidth;
+      numcopied = g_utf8_next_char(ptr) - in->textbuff;
+    } else {
+      /* Just update col. We'll append later. */
+      if (c == '\n') {
+        col = start;
+      } else if (!owl_fmtext_is_format_char(c)) {
+        col += mk_wcwidth(c);
+      }
+    }
+  }
+  /* Append anything we've missed. */
+  if (numcopied < in->textlen)
+    _owl_fmtext_append_fmtext(out, in, numcopied, in->textlen);
+}
+
 /* start with line 'aline' (where the first line is 0) and print
  * 'lines' number of lines into 'out'
  */
-int owl_fmtext_truncate_lines(owl_fmtext *in, int aline, int lines, owl_fmtext *out)
+int owl_fmtext_truncate_lines(const owl_fmtext *in, int aline, int lines, owl_fmtext *out)
 {
-  char *ptr1, *ptr2;
+  const char *ptr1, *ptr2;
   int i, offset;
   
   /* find the starting line */
@@ -416,26 +438,21 @@ int owl_fmtext_truncate_lines(owl_fmtext *in, int aline, int lines, owl_fmtext *
     offset = ptr1 - in->textbuff;
     ptr2 = strchr(ptr1, '\n');
     if (!ptr2) {
-      _owl_fmtext_append_fmtext(out, in, offset, (in->textlen) - 1);
+      /* Copy to the end of the buffer. */
+      _owl_fmtext_append_fmtext(out, in, offset, in->textlen);
       return(-1);
     }
-    _owl_fmtext_append_fmtext(out, in, offset, (ptr2 - ptr1) + offset);
+    /* Copy up to, and including, the new line. */
+    _owl_fmtext_append_fmtext(out, in, offset, (ptr2 - ptr1) + offset + 1);
     ptr1 = ptr2 + 1;
   }
   return(0);
 }
 
-/* Truncate the message so that each line begins at column 'acol' and
- * ends at 'bcol' or sooner.  The first column is number 0.  The new
- * message is placed in 'out'.  The message is * expected to end in a
- * new line for now. NOTE: This needs to be modified to deal with
- * backing up if we find a SPACING COMBINING MARK at the end of a
- * line. If that happens, we should back up to the last non-mark
- * character and stop there.
- */
-void owl_fmtext_truncate_cols(owl_fmtext *in, int acol, int bcol, owl_fmtext *out)
+/* Implementation of owl_fmtext_truncate_cols. Does not support tabs in input. */
+void _owl_fmtext_truncate_cols_internal(const owl_fmtext *in, int acol, int bcol, owl_fmtext *out)
 {
-  char *ptr_s, *ptr_e, *ptr_c, *last;
+  const char *ptr_s, *ptr_e, *ptr_c, *last;
   int col, st, padding, chwidth;
 
   /* copy the default attributes */
@@ -485,20 +502,19 @@ void owl_fmtext_truncate_cols(owl_fmtext *in, int acol, int bcol, owl_fmtext *ou
       /* lead padding */
       owl_fmtext_append_spaces(out, padding);
       if (ptr_c == ptr_e) {
-	/* We made it to the newline. */
-	_owl_fmtext_append_fmtext(out, in, ptr_s - in->textbuff, ptr_c - in->textbuff);
+	/* We made it to the newline. Append up to, and including it. */
+	_owl_fmtext_append_fmtext(out, in, ptr_s - in->textbuff, ptr_c - in->textbuff + 1);
       }
       else if (chwidth > 1) {
         /* Last char is wide, truncate. */
-        _owl_fmtext_append_fmtext(out, in, ptr_s - in->textbuff, ptr_c - in->textbuff - 1);
+        _owl_fmtext_append_fmtext(out, in, ptr_s - in->textbuff, ptr_c - in->textbuff);
         owl_fmtext_append_normal(out, "\n");
       }
       else {
-        /* Last char fits perfectly, We skip to the next char and back
-         * up a byte to make sure we get it all.
-         */
+        /* Last char fits perfectly, We stop at the next char to make
+	 * sure we get it all. */
         ptr_c = g_utf8_next_char(ptr_c);
-        _owl_fmtext_append_fmtext(out, in, ptr_s - in->textbuff, ptr_c - in->textbuff - 1);
+        _owl_fmtext_append_fmtext(out, in, ptr_s - in->textbuff, ptr_c - in->textbuff);
       }
     }
     else {
@@ -508,29 +524,105 @@ void owl_fmtext_truncate_cols(owl_fmtext *in, int acol, int bcol, owl_fmtext *ou
   }
 }
 
+/* Truncate the message so that each line begins at column 'acol' and
+ * ends at 'bcol' or sooner.  The first column is number 0.  The new
+ * message is placed in 'out'.  The message is expected to end in a
+ * new line for now.
+ *
+ * NOTE: This needs to be modified to deal with backing up if we find
+ * a SPACING COMBINING MARK at the end of a line. If that happens, we
+ * should back up to the last non-mark character and stop there.
+ *
+ * NOTE: If a line ends at bcol, we omit the newline. This is so printing
+ * to ncurses works.
+ */
+void owl_fmtext_truncate_cols(const owl_fmtext *in, int acol, int bcol, owl_fmtext *out)
+{
+  owl_fmtext notabs;
+
+  /* _owl_fmtext_truncate_cols_internal cannot handle tabs. */
+  if (strchr(in->textbuff, '\t')) {
+    owl_fmtext_init_null(&notabs);
+    owl_fmtext_expand_tabs(in, &notabs, 0);
+    _owl_fmtext_truncate_cols_internal(&notabs, acol, bcol, out);
+    owl_fmtext_cleanup(&notabs);
+  } else {
+    _owl_fmtext_truncate_cols_internal(in, acol, bcol, out);
+  }
+}
+
 /* Return the number of lines in 'f' */
-int owl_fmtext_num_lines(owl_fmtext *f)
+int owl_fmtext_num_lines(const owl_fmtext *f)
 {
   int lines, i;
+  char *lastbreak, *p;
 
-  if (f->textlen==0) return(0);
-  
   lines=0;
+  lastbreak = f->textbuff;
   for (i=0; i<f->textlen; i++) {
-    if (f->textbuff[i]=='\n') lines++;
+    if (f->textbuff[i]=='\n') {
+      lastbreak = f->textbuff + i;
+      lines++;
+    }
   }
 
-  /* if the last char wasn't a \n there's one more line */
-  if (f->textbuff[i-1]!='\n') lines++;
+  /* Check if there's a trailing line; formatting characters don't count. */
+  for (p = g_utf8_next_char(lastbreak);
+       p < f->textbuff + f->textlen;
+       p = g_utf8_next_char(p)) {
+    if (!owl_fmtext_is_format_char(g_utf8_get_char(p))) {
+      lines++;
+      break;
+    }
+  }
 
   return(lines);
 }
 
-char *owl_fmtext_get_text(owl_fmtext *f)
+/* Returns the line number, starting at 0, of the character which
+ * contains the byte at 'offset'. Note that a trailing newline is part
+ * of the line it ends. Also, while a trailing line of formatting
+ * characters does not contribute to owl_fmtext_num_lines, those
+ * characters are considered on a new line. */
+int owl_fmtext_line_number(const owl_fmtext *f, int offset)
+{
+  int i, lineno = 0;
+  if (offset >= f->textlen)
+    offset = f->textlen - 1;
+  for (i = 0; i < offset; i++) {
+    if (f->textbuff[i] == '\n')
+      lineno++;
+  }
+  return lineno;
+}
+
+/* Searches for line 'lineno' in 'f'. The returned range, [start,
+ * end), forms a half-open interval for the extent of the line. */
+void owl_fmtext_line_extents(const owl_fmtext *f, int lineno, int *o_start, int *o_end)
+{
+  int start, end;
+  char *newline;
+  for (start = 0; lineno > 0 && start < f->textlen; start++) {
+    if (f->textbuff[start] == '\n')
+      lineno--;
+  }
+  newline = strchr(f->textbuff + start, '\n');
+  /* Include the newline, if it is there. */
+  end = newline ? newline - f->textbuff + 1 : f->textlen;
+  if (o_start) *o_start = start;
+  if (o_end) *o_end = end;
+}
+
+const char *owl_fmtext_get_text(const owl_fmtext *f)
 {
   return(f->textbuff);
 }
 
+int owl_fmtext_num_bytes(const owl_fmtext *f)
+{
+  return(f->textlen);
+}
+
 /* set the charater at 'index' to be 'char'.  If index is out of
  * bounds don't do anything. If c or char at index is not ASCII, don't
  * do anything because it's not UTF-8 safe. */
@@ -543,7 +635,7 @@ void owl_fmtext_set_char(owl_fmtext *f, int index, char ch)
 }
 
 /* Make a copy of the fmtext 'src' into 'dst' */
-void owl_fmtext_copy(owl_fmtext *dst, owl_fmtext *src)
+void owl_fmtext_copy(owl_fmtext *dst, const owl_fmtext *src)
 {
   int mallocsize;
 
@@ -561,23 +653,28 @@ void owl_fmtext_copy(owl_fmtext *dst, owl_fmtext *src)
   dst->default_bgcolor = src->default_bgcolor;
 }
 
-/* return 1 if the string is found, 0 if not.  This is a case
- *  insensitive search.
+/* Search 'f' for the regex 're' for matches starting at
+ * 'start'. Returns the offset of the first match, -1 if not
+ * found. This is a case-insensitive search.
  */
-int owl_fmtext_search(owl_fmtext *f, char *string)
+int owl_fmtext_search(const owl_fmtext *f, const owl_regex *re, int start)
 {
-  if (stristr(f->textbuff, string)) return(1);
-  return(0);
+  int offset;
+  if (start > f->textlen ||
+      owl_regex_compare(re, f->textbuff + start, &offset, NULL) != 0)
+    return -1;
+  return offset + start;
 }
 
 
 /* Append the text 'text' to 'f' and interpret the zephyr style
  * formatting syntax to set appropriate attributes.
  */
-void owl_fmtext_append_ztext(owl_fmtext *f, char *text)
+void owl_fmtext_append_ztext(owl_fmtext *f, const char *text)
 {
   int stacksize, curattrs, curcolor;
-  char *ptr, *txtptr, *buff, *tmpptr;
+  const char *ptr, *txtptr, *tmpptr;
+  char *buff;
   int attrstack[32], chrstack[32], colorstack[32];
 
   curattrs=OWL_FMTEXT_ATTR_NONE;
@@ -696,7 +793,8 @@ void owl_fmtext_append_ztext(owl_fmtext *f, char *text)
 
           /* set it as the current color */
           curcolor=owl_util_string_to_color(buff);
-          if (curcolor==-1) curcolor=OWL_COLOR_DEFAULT;
+          if (curcolor == OWL_COLOR_INVALID)
+	      curcolor = OWL_COLOR_DEFAULT;
           owl_free(buff);
           txtptr=tmpptr+1;
           continue;
@@ -778,15 +876,15 @@ void owl_fmtext_append_ztext(owl_fmtext *f, char *text)
  * joins the elements together with join_with. 
  * If format_fn is specified, passes it the list element value
  * and it will return a string which this needs to free. */
-void owl_fmtext_append_list(owl_fmtext *f, owl_list *l, char *join_with, char *(format_fn)(char *))
+void owl_fmtext_append_list(owl_fmtext *f, const owl_list *l, const char *join_with, char *(format_fn)(const char *))
 {
   int i, size;
-  char *elem;
+  const char *elem;
   char *text;
 
   size = owl_list_get_size(l);
   for (i=0; i<size; i++) {
-    elem = (char*)owl_list_get_element(l,i);
+    elem = owl_list_get_element(l,i);
     if (elem && format_fn) {
       text = format_fn(elem);
       if (text) {
@@ -803,7 +901,7 @@ void owl_fmtext_append_list(owl_fmtext *f, owl_list *l, char *join_with, char *(
 }
 
 /* Free all memory allocated by the object */
-void owl_fmtext_free(owl_fmtext *f)
+void owl_fmtext_cleanup(owl_fmtext *f)
 {
   if (f->textbuff) owl_free(f->textbuff);
 }
@@ -812,13 +910,24 @@ void owl_fmtext_free(owl_fmtext *f)
 void owl_fmtext_init_colorpair_mgr(owl_colorpair_mgr *cpmgr)
 {
   /* This could be a bitarray if we wanted to save memory. */
-  short i, j;
-  cpmgr->next = 8;
-  
+  short i;
   /* The test is <= because we allocate COLORS+1 entries. */
   cpmgr->pairs = owl_malloc((COLORS+1) * sizeof(short*));
   for(i = 0; i <= COLORS; i++) {
     cpmgr->pairs[i] = owl_malloc((COLORS+1) * sizeof(short));
+  }
+  owl_fmtext_reset_colorpairs(cpmgr);
+}
+
+/* Reset used list */
+void owl_fmtext_reset_colorpairs(owl_colorpair_mgr *cpmgr)
+{
+  short i, j;
+
+  cpmgr->overflow = false;
+  cpmgr->next = 8;
+  /* The test is <= because we allocated COLORS+1 entries. */
+  for(i = 0; i <= COLORS; i++) {
     for(j = 0; j <= COLORS; j++) {
       cpmgr->pairs[i][j] = -1;
     }
@@ -833,34 +942,11 @@ void owl_fmtext_init_colorpair_mgr(owl_colorpair_mgr *cpmgr)
   }
 }
 
-/* Reset used list */
-void owl_fmtext_reset_colorpairs()
-{
-  if (owl_global_get_hascolors(&g)) {
-    short i, j;
-    owl_colorpair_mgr *cpmgr = owl_global_get_colorpair_mgr(&g);
-    cpmgr->next = 8;
-    
-    /* The test is <= because we allocated COLORS+1 entries. */
-    for(i = 0; i <= COLORS; i++) {
-      for(j = 0; j <= COLORS; j++) {
-	cpmgr->pairs[i][j] = -1;
-      }
-    }
-    for(i = 0; i < 8; i++) {
-      short fg, bg;
-      if (i >= COLORS) continue;
-      pair_content(i, &fg, &bg);
-      cpmgr->pairs[fg+1][bg+1] = i;
-    }
-  }
-}
-
 /* Assign pairs by request */
 short owl_fmtext_get_colorpair(int fg, int bg)
 {
   owl_colorpair_mgr *cpmgr;
-  short pair, default_bg;
+  short pair;
 
   /* Sanity (Bounds) Check */
   if (fg > COLORS || fg < OWL_COLOR_DEFAULT) fg = OWL_COLOR_DEFAULT;
@@ -868,11 +954,9 @@ short owl_fmtext_get_colorpair(int fg, int bg)
 	    
 #ifdef HAVE_USE_DEFAULT_COLORS
   if (fg == OWL_COLOR_DEFAULT) fg = -1;
-  default_bg = OWL_COLOR_DEFAULT;
 #else
   if (fg == OWL_COLOR_DEFAULT) fg = 0;
   if (bg == OWL_COLOR_DEFAULT) bg = 0;
-  default_bg = COLOR_BLACK;
 #endif
 
   /* looking for a pair we already set up for this draw. */
@@ -890,11 +974,13 @@ short owl_fmtext_get_colorpair(int fg, int bg)
     else if (bg != OWL_COLOR_DEFAULT) {
       /* We still don't have a pair, drop the background color. Too bad. */
       owl_function_debugmsg("colorpairs: color shortage - dropping background color.");
+      cpmgr->overflow = true;
       pair = owl_fmtext_get_colorpair(fg, OWL_COLOR_DEFAULT);
     }
     else {
       /* We still don't have a pair, defaults all around. */
       owl_function_debugmsg("colorpairs: color shortage - dropping foreground and background color.");
+      cpmgr->overflow = true;
       pair = 0;
     }
   }
