@@ -8,19 +8,18 @@
 #include <time.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id$";
-
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 256
 #endif
 
+static void _owl_global_init_windows(owl_global *g);
+
 void owl_global_init(owl_global *g) {
   struct hostent *hent;
   char hostname[MAXHOSTNAMELEN];
   char *cd;
 
-  g->malloced=0;
-  g->freed=0;
+  g_type_init();
 
   gethostname(hostname, MAXHOSTNAMELEN);
   hent=gethostbyname(hostname);
@@ -30,34 +29,38 @@ void owl_global_init(owl_global *g) {
     g->thishost=owl_strdup(hent->h_name);
   }
 
-  owl_context_init(&g->ctx);
-  owl_context_set_startup(&g->ctx);
+  g->lines=LINES;
+  g->cols=COLS;
+  /* We shouldn't need this if we initialize lines and cols before the first
+   * owl_window_get_screen, but to be safe, we synchronize. */
+  owl_window_resize(owl_window_get_screen(), g->lines, g->cols);
+
+  g->context_stack = NULL;
+  owl_global_push_context(g, OWL_CTX_STARTUP, NULL, NULL, NULL);
+
   g->curmsg=0;
   g->topmsg=0;
   g->markedmsgid=-1;
-  g->needrefresh=1;
   g->startupargs=NULL;
 
   owl_variable_dict_setup(&(g->vars));
-  owl_cmddict_setup(&(g->cmds));
-
-  g->lines=LINES;
-  g->cols=COLS;
 
   g->rightshift=0;
 
-  owl_editwin_init(&(g->tw), NULL, owl_global_get_typwin_lines(g), g->cols, OWL_EDITWIN_STYLE_ONELINE, NULL);
+  g->pw = NULL;
+  g->vw = NULL;
+  g->tw = NULL;
 
   owl_keyhandler_init(&g->kh);
   owl_keys_setup_keymaps(&g->kh);
 
-  owl_list_create(&(g->filterlist));
+  owl_dict_create(&(g->filters));
+  g->filterlist = NULL;
   owl_list_create(&(g->puntlist));
-  owl_list_create(&(g->messagequeue));
+  g->messagequeue = g_queue_new();
   owl_dict_create(&(g->styledict));
   g->curmsg_vert_offset=0;
   g->resizepending=0;
-  g->typwinactive=0;
   g->direction=OWL_DIRECTION_DOWNWARDS;
   g->zaway=0;
   if (has_colors()) {
@@ -66,22 +69,18 @@ void owl_global_init(owl_global *g) {
   g->colorpairs=COLOR_PAIRS;
   owl_fmtext_init_colorpair_mgr(&(g->cpmgr));
   g->debug=OWL_DEBUG;
-  g->searchactive=0;
-  g->searchstring=NULL;
+  owl_regex_init(&g->search_re);
   g->starttime=time(NULL); /* assumes we call init only a start time */
   g->lastinputtime=g->starttime;
   g->newmsgproc_pid=0;
   
   owl_global_set_config_format(g, 0);
-  owl_global_set_userclue(g, OWL_USERCLUE_NONE);
   owl_global_set_no_have_config(g);
   owl_history_init(&(g->msghist));
   owl_history_init(&(g->cmdhist));
   owl_history_set_norepeats(&(g->cmdhist));
   g->nextmsgid=0;
 
-  _owl_global_setup_windows(g);
-
   /* Fill in some variables which don't have constant defaults */
   /* TODO: come back later and check passwd file first */
   g->homedir=owl_strdup(getenv("HOME"));
@@ -93,8 +92,8 @@ void owl_global_init(owl_global *g) {
   owl_free(cd);
 
   owl_messagelist_create(&(g->msglist));
-  owl_mainwin_init(&(g->mw));
-  owl_popwin_init(&(g->pw));
+
+  _owl_global_init_windows(g);
 
   g->aim_screenname=NULL;
   g->aim_screenname_for_filters=NULL;
@@ -111,74 +110,113 @@ void owl_global_init(owl_global *g) {
 
   owl_zbuddylist_create(&(g->zbuddies));
 
-  owl_obarray_init(&(g->obarray));
+  g->zaldlist = NULL;
+  g->pseudologin_notify = 0;
 
   owl_message_init_fmtext_cache();
-  owl_list_create(&(g->dispatchlist));
+  owl_list_create(&(g->io_dispatch_list));
+  owl_list_create(&(g->psa_list));
   g->timerlist = NULL;
   g->interrupted = FALSE;
 }
 
-void _owl_global_setup_windows(owl_global *g) {
-  int cols, typwin_lines;
+static void _owl_global_init_windows(owl_global *g)
+{
+  /* Create the main window */
+  owl_mainpanel_init(&(g->mainpanel));
 
-  cols=g->cols;
-  typwin_lines=owl_global_get_typwin_lines(g);
+  /* Create the widgets */
+  owl_mainwin_init(&(g->mw), g->mainpanel.recwin);
+  owl_msgwin_init(&(g->msgwin), g->mainpanel.msgwin);
+  owl_sepbar_init(g->mainpanel.sepwin);
 
-  /* set the new window sizes */
-  g->recwinlines=g->lines-(typwin_lines+2);
-  if (g->recwinlines<0) {
-    /* gotta deal with this */
-    g->recwinlines=0;
-  }
+  owl_window_set_default_cursor(g->mainpanel.sepwin);
 
-  owl_function_debugmsg("_owl_global_setup_windows: about to call newwin(%i, %i, 0, 0)\n", g->recwinlines, cols);
-
-  /* create the new windows */
-  g->recwin=newwin(g->recwinlines, cols, 0, 0);
-  if (g->recwin==NULL) {
-    owl_function_debugmsg("_owl_global_setup_windows: newwin returned NULL\n");
-    endwin();
-    exit(50);
-  }
-      
-  g->sepwin=newwin(1, cols, g->recwinlines, 0);
-  g->msgwin=newwin(1, cols, g->recwinlines+1, 0);
-  g->typwin=newwin(typwin_lines, cols, g->recwinlines+2, 0);
-
-  owl_editwin_set_curswin(&(g->tw), g->typwin, typwin_lines, g->cols);
-
-  idlok(g->typwin, FALSE);
-  idlok(g->recwin, FALSE);
-  idlok(g->sepwin, FALSE);
-  idlok(g->msgwin, FALSE);
-
-  nodelay(g->typwin, 1);
-  keypad(g->typwin, TRUE);
-  wmove(g->typwin, 0, 0);
-
-  meta(g->typwin, TRUE);
+  /* set up a pad for input */
+  g->input_pad = newpad(1, 1);
+  nodelay(g->input_pad, 1);
+  keypad(g->input_pad, 1);
+  meta(g->input_pad, 1);
 }
 
-owl_context *owl_global_get_context(owl_global *g) {
-  return(&g->ctx);
+void owl_global_sepbar_dirty(owl_global *g)
+{
+  owl_window_dirty(g->mainpanel.sepwin);
 }
-			 
-int owl_global_get_lines(owl_global *g) {
+
+/* Called once perl has been initialized */
+void owl_global_complete_setup(owl_global *g)
+{
+  owl_cmddict_setup(&(g->cmds));
+}
+
+owl_context *owl_global_get_context(const owl_global *g) {
+  if (!g->context_stack)
+    return NULL;
+  return g->context_stack->data;
+}
+
+static void owl_global_activate_context(owl_global *g, owl_context *c) {
+  if (!c)
+    return;
+
+  if (c->keymap) {
+    if (!owl_keyhandler_activate(owl_global_get_keyhandler(g), c->keymap)) {
+      owl_function_error("Unable to activate keymap '%s'", c->keymap);
+    }
+  }
+  owl_window_set_cursor(c->cursor);
+}
+
+void owl_global_push_context(owl_global *g, int mode, void *data, const char *keymap, owl_window *cursor) {
+  owl_context *c;
+  c = owl_context_new(mode, data, keymap, cursor);
+  owl_global_push_context_obj(g, c);
+}
+
+void owl_global_push_context_obj(owl_global *g, owl_context *c)
+{
+  g->context_stack = g_list_prepend(g->context_stack, c);
+  owl_global_activate_context(g, owl_global_get_context(g));
+}
+
+/* Pops the current context from the context stack and returns it. Caller is
+ * responsible for freeing. */
+owl_context *owl_global_pop_context_no_delete(owl_global *g) {
+  owl_context *c;
+  if (!g->context_stack)
+    return NULL;
+  c = owl_global_get_context(g);
+  owl_context_deactivate(c);
+  g->context_stack = g_list_delete_link(g->context_stack,
+                                        g->context_stack);
+  owl_global_activate_context(g, owl_global_get_context(g));
+  return c;
+}
+
+/* Pops the current context from the context stack and deletes it. */
+void owl_global_pop_context(owl_global *g) {
+  owl_context *c;
+  c = owl_global_pop_context_no_delete(g);
+  if (c)
+    owl_context_delete(c);
+}
+
+int owl_global_get_lines(const owl_global *g) {
   return(g->lines);
 }
 
-int owl_global_get_cols(owl_global *g) {
+int owl_global_get_cols(const owl_global *g) {
   return(g->cols);
 }
 
-int owl_global_get_recwin_lines(owl_global *g) {
-  return(g->recwinlines);
+int owl_global_get_recwin_lines(const owl_global *g) {
+  return g->mainpanel.recwinlines;
 }
 
 /* curmsg */
 
-int owl_global_get_curmsg(owl_global *g) {
+int owl_global_get_curmsg(const owl_global *g) {
   return(g->curmsg);
 }
 
@@ -191,7 +229,7 @@ void owl_global_set_curmsg(owl_global *g, int i) {
 
 /* topmsg */
 
-int owl_global_get_topmsg(owl_global *g) {
+int owl_global_get_topmsg(const owl_global *g) {
   return(g->topmsg);
 }
 
@@ -201,14 +239,14 @@ void owl_global_set_topmsg(owl_global *g, int i) {
 
 /* markedmsgid */
 
-int owl_global_get_markedmsgid(owl_global *g) {
+int owl_global_get_markedmsgid(const owl_global *g) {
   return(g->markedmsgid);
 }
 
 void owl_global_set_markedmsgid(owl_global *g, int i) {
   g->markedmsgid=i;
   /* i; index of message in the current view.
-  owl_message *m;
+  const owl_message *m;
   owl_view *v;
 
   v = owl_global_get_current_view(&g);
@@ -224,7 +262,11 @@ owl_mainwin *owl_global_get_mainwin(owl_global *g) {
 }
 
 owl_popwin *owl_global_get_popwin(owl_global *g) {
-  return(&(g->pw));
+  return g->pw;
+}
+
+void owl_global_set_popwin(owl_global *g, owl_popwin *pw) {
+  g->pw = pw;
 }
 
 /* msglist */
@@ -239,61 +281,10 @@ owl_keyhandler *owl_global_get_keyhandler(owl_global *g) {
   return(&(g->kh));
 }
 
-/* curses windows */
-
-WINDOW *owl_global_get_curs_recwin(owl_global *g) {
-  return(g->recwin);
-}
-
-WINDOW *owl_global_get_curs_sepwin(owl_global *g) {
-  return(g->sepwin);
-}
-
-WINDOW *owl_global_get_curs_msgwin(owl_global *g) {
-  return(g->msgwin);
-}
-
-WINDOW *owl_global_get_curs_typwin(owl_global *g) {
-  return(g->typwin);
-}
-
-/* typwin */
-
-owl_editwin *owl_global_get_typwin(owl_global *g) {
-  return(&(g->tw));
-}
-
-/* buffercommand */
-
-void owl_global_set_buffercommand(owl_global *g, char *command) {
-  owl_editwin_set_command(owl_global_get_typwin(g), command);
-}
-
-char *owl_global_get_buffercommand(owl_global *g) {
-  return owl_editwin_get_command(owl_global_get_typwin(g));
-}
-
-void owl_global_set_buffercallback(owl_global *g, void (*cb)(owl_editwin*)) {
-  owl_editwin_set_callback(owl_global_get_typwin(g), cb);
-}
-
-void (*owl_global_get_buffercallback(owl_global *g))(owl_editwin*) {
-  return owl_editwin_get_callback(owl_global_get_typwin(g));
-}
-
-/* refresh */
-
-int owl_global_is_needrefresh(owl_global *g) {
-  if (g->needrefresh==1) return(1);
-  return(0);
-}
-
-void owl_global_set_needrefresh(owl_global *g) {
-  g->needrefresh=1;
-}
-
-void owl_global_set_noneedrefresh(owl_global *g) {
-  g->needrefresh=0;
+/* Gets the currently active typwin out of the current context. */
+owl_editwin *owl_global_current_typwin(const owl_global *g) {
+  owl_context *ctx = owl_global_get_context(g);
+  return owl_editcontext_get_editwin(ctx);
 }
 
 /* variable dictionary */
@@ -311,34 +302,53 @@ owl_cmddict *owl_global_get_cmddict(owl_global *g) {
 /* rightshift */
 
 void owl_global_set_rightshift(owl_global *g, int i) {
-  g->rightshift=i;
+  g->rightshift = i;
+  owl_mainwin_redisplay(owl_global_get_mainwin(g));
 }
 
-int owl_global_get_rightshift(owl_global *g) {
+int owl_global_get_rightshift(const owl_global *g) {
   return(g->rightshift);
 }
 
 /* typwin */
 
-int owl_global_is_typwin_active(owl_global *g) {
-  if (g->typwinactive==1) return(1);
-  return(0);
-}
-
-void owl_global_set_typwin_active(owl_global *g) {
-  int d = owl_global_get_typewindelta(g);
-  if (d > 0)
+owl_editwin *owl_global_set_typwin_active(owl_global *g, int style, owl_history *hist) {
+  int d;
+  d = owl_global_get_typewindelta(g);
+  if (d > 0 && style == OWL_EDITWIN_STYLE_MULTILINE)
       owl_function_resize_typwin(owl_global_get_typwin_lines(g) + d);
 
-  g->typwinactive=1;
+  if (g->typwin_erase_id) {
+    g_signal_handler_disconnect(g->mainpanel.typwin, g->typwin_erase_id);
+    g->typwin_erase_id = 0;
+  }
+
+  g->tw = owl_editwin_new(g->mainpanel.typwin,
+                          owl_global_get_typwin_lines(g),
+                          g->cols,
+                          style,
+                          hist);
+  return g->tw;
+}
+
+void owl_global_deactivate_editcontext(owl_context *ctx) {
+  owl_global *g = ctx->cbdata;
+  owl_global_set_typwin_inactive(g);
 }
 
 void owl_global_set_typwin_inactive(owl_global *g) {
   int d = owl_global_get_typewindelta(g);
-  if (d > 0)
+  if (d > 0 && owl_editwin_get_style(g->tw) == OWL_EDITWIN_STYLE_MULTILINE)
       owl_function_resize_typwin(owl_global_get_typwin_lines(g) - d);
 
-  g->typwinactive=0;
+  if (!g->typwin_erase_id) {
+    g->typwin_erase_id =
+      g_signal_connect(g->mainpanel.typwin, "redraw", G_CALLBACK(owl_window_erase_cb), NULL);
+  }
+  owl_window_dirty(g->mainpanel.typwin);
+
+  owl_editwin_unref(g->tw);
+  g->tw = NULL;
 }
 
 /* resize */
@@ -347,12 +357,12 @@ void owl_global_set_resize_pending(owl_global *g) {
   g->resizepending=1;
 }
 
-char *owl_global_get_homedir(owl_global *g) {
+const char *owl_global_get_homedir(const owl_global *g) {
   if (g->homedir) return(g->homedir);
   return("/");
 }
 
-char *owl_global_get_confdir(owl_global *g) {
+const char *owl_global_get_confdir(const owl_global *g) {
   if (g->confdir) return(g->confdir);
   return("/");
 }
@@ -360,19 +370,19 @@ char *owl_global_get_confdir(owl_global *g) {
 /*
  * Setting this also sets startupfile to confdir/startup
  */
-void owl_global_set_confdir(owl_global *g, char *cd) {
-  free(g->confdir);
+void owl_global_set_confdir(owl_global *g, const char *cd) {
+  owl_free(g->confdir);
   g->confdir = owl_strdup(cd);
-  free(g->startupfile);
+  owl_free(g->startupfile);
   g->startupfile = owl_sprintf("%s/startup", cd);
 }
 
-char *owl_global_get_startupfile(owl_global *g) {
+const char *owl_global_get_startupfile(const owl_global *g) {
   if(g->startupfile) return(g->startupfile);
   return("/");
 }
 
-int owl_global_get_direction(owl_global *g) {
+int owl_global_get_direction(const owl_global *g) {
   return(g->direction);
 }
 
@@ -390,11 +400,11 @@ void owl_global_set_perlinterp(owl_global *g, void *p) {
   g->perl=p;
 }
 
-void *owl_global_get_perlinterp(owl_global *g) {
+void *owl_global_get_perlinterp(const owl_global *g) {
   return(g->perl);
 }
 
-int owl_global_is_config_format(owl_global *g) {
+int owl_global_is_config_format(const owl_global *g) {
   if (g->config_format) return(1);
   return(0);
 }
@@ -420,96 +430,56 @@ int owl_global_have_config(owl_global *g) {
   return(0);
 }
 
-void owl_global_resize(owl_global *g, int x, int y) {
-  /* resize the screen.  If x or y is 0 use the terminal size */
+/*
+ * Compute the size of the terminal. Try a ioctl, fallback to other stuff on
+ * fail.
+ */
+void owl_global_get_terminal_size(int *lines, int *cols) {
   struct winsize size;
-    
-  if (!g->resizepending) return;
-
-  /* delete the current windows */
-  delwin(g->recwin);
-  delwin(g->sepwin);
-  delwin(g->msgwin);
-  delwin(g->typwin);
-  if (!isendwin()) {
-    endwin();
-  }
-
-  refresh();
-
   /* get the new size */
   ioctl(STDIN_FILENO, TIOCGWINSZ, &size);
-  if (x==0) {
-    if (size.ws_row) {
-      g->lines=size.ws_row;
-    } else {
-      g->lines=LINES;
-    } 
+  if (size.ws_row) {
+    *lines = size.ws_row;
   } else {
-      g->lines=x;
+    *lines = LINES;
   }
 
-  if (y==0) {
-    if (size.ws_col) {
-      g->cols=size.ws_col;
-    } else {
-      g->cols=COLS;
-    } 
+  if (size.ws_col) {
+    *cols = size.ws_col;
   } else {
-    g->cols=y;
+    *cols = COLS;
   }
+}
 
-#ifdef HAVE_RESIZETERM
-  resizeterm(size.ws_row, size.ws_col);
-#endif
+void owl_global_check_resize(owl_global *g) {
+  /* resize the screen.  If lines or cols is 0 use the terminal size */
+  if (!g->resizepending) return;
+  g->resizepending = 0;
 
-  /* re-initialize the windows */
-  _owl_global_setup_windows(g);
+  owl_global_get_terminal_size(&g->lines, &g->cols);
+  owl_window_resize(owl_window_get_screen(), g->lines, g->cols);
 
-  /* in case any styles rely on the current width */
-  owl_messagelist_invalidate_formats(owl_global_get_msglist(g));
-
-  /* recalculate the topmsg to make sure the current message is on
-   * screen */
-  owl_function_calculate_topmsg(OWL_DIRECTION_NONE);
-
-  /* refresh stuff */
-  g->needrefresh=1;
-  owl_mainwin_redisplay(&(g->mw));
-  sepbar(NULL);
-  owl_editwin_redisplay(&(g->tw), 0);
-  owl_function_full_redisplay(&g);
-
-  /* TODO: this should handle other forms of popwins */
-  if (owl_popwin_is_active(owl_global_get_popwin(g)) 
-      && owl_global_get_viewwin(g)) {
-    owl_popwin_refresh(owl_global_get_popwin(g));
-    owl_viewwin_redisplay(owl_global_get_viewwin(g), 0);
-  }
-
-  owl_function_debugmsg("New size is %i lines, %i cols.", size.ws_row, size.ws_col);
-  owl_function_makemsg("");
-  g->resizepending=0;
+  owl_function_debugmsg("New size is %i lines, %i cols.", g->lines, g->cols);
 }
 
 /* debug */
 
-int owl_global_is_debug_fast(owl_global *g) {
+int owl_global_is_debug_fast(const owl_global *g) {
   if (g->debug) return(1);
   return(0);
 }
 
 /* starttime */
 
-time_t owl_global_get_starttime(owl_global *g) {
+time_t owl_global_get_starttime(const owl_global *g) {
   return(g->starttime);
 }
 
-time_t owl_global_get_runtime(owl_global *g) {
+time_t owl_global_get_runtime(const owl_global *g) {
   return(time(NULL)-g->starttime);
 }
 
-time_t owl_global_get_lastinputtime(owl_global *g) {
+time_t owl_global_get_lastinputtime(const owl_global *g) {
   return(g->lastinputtime);
 }
 
@@ -517,63 +487,39 @@ void owl_global_set_lastinputtime(owl_global *g, time_t time) {
   g->lastinputtime = time;
 }
 
-time_t owl_global_get_idletime(owl_global *g) {
+time_t owl_global_get_idletime(const owl_global *g) {
   return(time(NULL)-g->lastinputtime);
 }
 
-void owl_global_get_runtime_string(owl_global *g, char *buff) {
-  time_t diff;
-
-  diff=time(NULL)-owl_global_get_starttime(g);
-
-  /* print something nicer later */   
-  sprintf(buff, "%i seconds", (int) diff);
-}
-
-char *owl_global_get_hostname(owl_global *g) {
+const char *owl_global_get_hostname(const owl_global *g) {
   if (g->thishost) return(g->thishost);
   return("");
 }
 
-/* userclue */
-
-void owl_global_set_userclue(owl_global *g, int clue) {
-  g->userclue=clue;
-}
-
-void owl_global_add_userclue(owl_global *g, int clue) {
-  g->userclue|=clue;
-}
-
-int owl_global_get_userclue(owl_global *g) {
-  return(g->userclue);
-}
-
-int owl_global_is_userclue(owl_global *g, int clue) {
-  if (g->userclue & clue) return(1);
-  return(0);
-}
-
 /* viewwin */
 
 owl_viewwin *owl_global_get_viewwin(owl_global *g) {
-  return(&(g->vw));
+  return g->vw;
+}
+
+void owl_global_set_viewwin(owl_global *g, owl_viewwin *vw) {
+  g->vw = vw;
 }
 
 
 /* vert offset */
 
-int owl_global_get_curmsg_vert_offset(owl_global *g) {
+int owl_global_get_curmsg_vert_offset(const owl_global *g) {
   return(g->curmsg_vert_offset);
 }
 
 void owl_global_set_curmsg_vert_offset(owl_global *g, int i) {
-  g->curmsg_vert_offset=i;
+  g->curmsg_vert_offset = i;
 }
 
 /* startup args */
 
-void owl_global_set_startupargs(owl_global *g, int argc, char **argv) {
+void owl_global_set_startupargs(owl_global *g, int argc, const char *const *argv) {
   int i, len;
 
   if (g->startupargs) owl_free(g->startupargs);
@@ -591,7 +537,7 @@ void owl_global_set_startupargs(owl_global *g, int argc, char **argv) {
   g->startupargs[strlen(g->startupargs)-1]='\0';
 }
 
-char *owl_global_get_startupargs(owl_global *g) {
+const char *owl_global_get_startupargs(const owl_global *g) {
   if (g->startupargs) return(g->startupargs);
   return("");
 }
@@ -607,42 +553,39 @@ owl_history *owl_global_get_cmd_history(owl_global *g) {
 }
 
 /* filterlist */
+typedef struct _owl_global_filter_ent {         /* noproto */
+  owl_global *g;
+  owl_filter *f;
+} owl_global_filter_ent;
 
-owl_list *owl_global_get_filterlist(owl_global *g) {
-  return(&(g->filterlist));
+owl_filter *owl_global_get_filter(const owl_global *g, const char *name) {
+  owl_global_filter_ent *e = owl_dict_find_element(&(g->filters), name);
+  if (e) return e->f;
+  return NULL;
 }
 
-owl_filter *owl_global_get_filter(owl_global *g, char *name) {
-  int i, j;
-  owl_filter *f;
-
-  j=owl_list_get_size(&(g->filterlist));
-  for (i=0; i<j; i++) {
-    f=owl_list_get_element(&(g->filterlist), i);
-    if (!strcmp(name, owl_filter_get_name(f))) {
-      return(f);
-    }
-  }
-  return(NULL);
+static void owl_global_delete_filter_ent(void *data)
+{
+  owl_global_filter_ent *e = data;
+  e->g->filterlist = g_list_remove(e->g->filterlist, e->f);
+  owl_filter_delete(e->f);
+  owl_free(e);
 }
 
 void owl_global_add_filter(owl_global *g, owl_filter *f) {
-  owl_list_append_element(&(g->filterlist), f);
+  owl_global_filter_ent *e = owl_malloc(sizeof *e);
+  e->g = g;
+  e->f = f;
+
+  owl_dict_insert_element(&(g->filters), owl_filter_get_name(f),
+                          e, owl_global_delete_filter_ent);
+  g->filterlist = g_list_append(g->filterlist, f);
 }
 
-void owl_global_remove_filter(owl_global *g, char *name) {
-  int i, j;
-  owl_filter *f;
-
-  j=owl_list_get_size(&(g->filterlist));
-  for (i=0; i<j; i++) {
-    f=owl_list_get_element(&(g->filterlist), i);
-    if (!strcmp(name, owl_filter_get_name(f))) {
-      owl_filter_free(f);
-      owl_list_remove_element(&(g->filterlist), i);
-      break;
-    }
-  }
+void owl_global_remove_filter(owl_global *g, const char *name) {
+  owl_global_filter_ent *e = owl_dict_remove_element(&(g->filters), name);
+  if (e)
+    owl_global_delete_filter_ent(e);
 }
 
 /* nextmsgid */
@@ -659,14 +602,14 @@ owl_view *owl_global_get_current_view(owl_global *g) {
 
 /* has colors */
 
-int owl_global_get_hascolors(owl_global *g) {
+int owl_global_get_hascolors(const owl_global *g) {
   if (g->hascolors) return(1);
   return(0);
 }
 
 /* color pairs */
 
-int owl_global_get_colorpairs(owl_global *g) {
+int owl_global_get_colorpairs(const owl_global *g) {
   return(g->colorpairs);
 }
 
@@ -680,8 +623,8 @@ owl_list *owl_global_get_puntlist(owl_global *g) {
   return(&(g->puntlist));
 }
 
-int owl_global_message_is_puntable(owl_global *g, owl_message *m) {
-  owl_list *pl;
+int owl_global_message_is_puntable(owl_global *g, const owl_message *m) {
+  const owl_list *pl;
   int i, j;
 
   pl=owl_global_get_puntlist(g);
@@ -693,7 +636,7 @@ int owl_global_message_is_puntable(owl_global *g, owl_message *m) {
 }
 
 int owl_global_should_followlast(owl_global *g) {
-  owl_view *v;
+  const owl_view *v;
   
   if (!owl_global_is__followlast(g)) return(0);
   
@@ -703,43 +646,45 @@ int owl_global_should_followlast(owl_global *g) {
   return(0);
 }
 
-int owl_global_is_search_active(owl_global *g) {
-  if (g->searchactive) return(1);
+int owl_global_is_search_active(const owl_global *g) {
+  if (owl_regex_is_set(&g->search_re)) return(1);
   return(0);
 }
 
-void owl_global_set_search_active(owl_global *g, char *string) {
-  g->searchactive=1;
-  if (g->searchstring != NULL) owl_free(g->searchstring);
-  g->searchstring=owl_strdup(string);
+void owl_global_set_search_re(owl_global *g, const owl_regex *re) {
+  if (owl_regex_is_set(&g->search_re)) {
+    owl_regex_cleanup(&g->search_re);
+    owl_regex_init(&g->search_re);
+  }
+  if (re != NULL)
+    owl_regex_copy(re, &g->search_re);
+  /* TODO: Emit a signal so we don't depend on the viewwin and mainwin */
+  if (owl_global_get_viewwin(g))
+    owl_viewwin_dirty(owl_global_get_viewwin(g));
+  owl_mainwin_redisplay(owl_global_get_mainwin(g));
 }
 
-void owl_global_set_search_inactive(owl_global *g) {
-  g->searchactive=0;
+const owl_regex *owl_global_get_search_re(const owl_global *g) {
+  return &g->search_re;
 }
 
-char *owl_global_get_search_string(owl_global *g) {
-  if (g->searchstring==NULL) return("");
-  return(g->searchstring);
-}
-
-void owl_global_set_newmsgproc_pid(owl_global *g, int i) {
+void owl_global_set_newmsgproc_pid(owl_global *g, pid_t i) {
   g->newmsgproc_pid=i;
 }
 
-int owl_global_get_newmsgproc_pid(owl_global *g) {
+pid_t owl_global_get_newmsgproc_pid(const owl_global *g) {
   return(g->newmsgproc_pid);
 }
 
 /* AIM stuff */
 
-int owl_global_is_aimloggedin(owl_global *g)
+int owl_global_is_aimloggedin(const owl_global *g)
 {
   if (g->aim_loggedin) return(1);
   return(0);
 }
 
-char *owl_global_get_aim_screenname(owl_global *g)
+const char *owl_global_get_aim_screenname(const owl_global *g)
 {
   if (owl_global_is_aimloggedin(g)) {
     return (g->aim_screenname);
@@ -747,7 +692,7 @@ char *owl_global_get_aim_screenname(owl_global *g)
   return("");
 }
 
-char *owl_global_get_aim_screenname_for_filters(owl_global *g)
+const char *owl_global_get_aim_screenname_for_filters(const owl_global *g)
 {
   if (owl_global_is_aimloggedin(g)) {
     return (g->aim_screenname_for_filters);
@@ -755,16 +700,15 @@ char *owl_global_get_aim_screenname_for_filters(owl_global *g)
   return("");
 }
 
-void owl_global_set_aimloggedin(owl_global *g, char *screenname)
+void owl_global_set_aimloggedin(owl_global *g, const char *screenname)
 {
-  char *sn_escaped, *quote;
+  char *sn_escaped;
   g->aim_loggedin=1;
   if (g->aim_screenname) owl_free(g->aim_screenname);
   if (g->aim_screenname_for_filters) owl_free(g->aim_screenname_for_filters);
   g->aim_screenname=owl_strdup(screenname);
   sn_escaped = owl_text_quote(screenname, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
-  quote = owl_getquoting(sn_escaped);
-  g->aim_screenname_for_filters=owl_sprintf("%s%s%s", quote, sn_escaped, quote);
+  g->aim_screenname_for_filters = owl_arg_quote(sn_escaped);
   owl_free(sn_escaped);
 }
 
@@ -773,7 +717,7 @@ void owl_global_set_aimnologgedin(owl_global *g)
   g->aim_loggedin=0;
 }
 
-int owl_global_is_doaimevents(owl_global *g)
+int owl_global_is_doaimevents(const owl_global *g)
 {
   if (g->aim_doprocessing) return(1);
   return(0);
@@ -808,7 +752,7 @@ void owl_global_set_bossconn(owl_global *g, aim_conn_t *conn)
 
 void owl_global_messagequeue_addmsg(owl_global *g, owl_message *m)
 {
-  owl_list_append_element(&(g->messagequeue), m);
+  g_queue_push_tail(g->messagequeue, m);
 }
 
 /* pop off the first message and return it.  Return NULL if the queue
@@ -819,16 +763,15 @@ owl_message *owl_global_messagequeue_popmsg(owl_global *g)
 {
   owl_message *out;
 
-  if (owl_list_get_size(&(g->messagequeue))==0) return(NULL);
-  out=owl_list_get_element(&(g->messagequeue), 0);
-  owl_list_remove_element(&(g->messagequeue), 0);
-  return(out);
+  if (g_queue_is_empty(g->messagequeue))
+    return NULL;
+  out = g_queue_pop_head(g->messagequeue);
+  return out;
 }
 
 int owl_global_messagequeue_pending(owl_global *g)
 {
-  if (owl_list_get_size(&(g->messagequeue))==0) return(0);
-  return(1);
+  return !g_queue_is_empty(g->messagequeue);
 }
 
 owl_buddylist *owl_global_get_buddylist(owl_global *g)
@@ -840,14 +783,14 @@ owl_buddylist *owl_global_get_buddylist(owl_global *g)
 
 /* Return the style with name 'name'.  If it does not exist return
  * NULL */
-owl_style *owl_global_get_style_by_name(owl_global *g, char *name)
+const owl_style *owl_global_get_style_by_name(const owl_global *g, const char *name)
 {
   return owl_dict_find_element(&(g->styledict), name);
 }
 
 /* creates a list and fills it in with keys.  duplicates the keys, 
  * so they will need to be freed by the caller. */
-int owl_global_get_style_names(owl_global *g, owl_list *l) {
+int owl_global_get_style_names(const owl_global *g, owl_list *l) {
   return owl_dict_get_keys(&(g->styledict), l);
 }
 
@@ -862,7 +805,7 @@ void owl_global_add_style(owl_global *g, owl_style *s)
                 owl_style_get_name(s)))
     g->current_view.style = s;
   owl_dict_insert_element(&(g->styledict), owl_style_get_name(s),
-                          s, (void(*)(void*))owl_style_free);
+                          s, (void (*)(void *))owl_style_delete);
 }
 
 void owl_global_set_haveaim(owl_global *g)
@@ -870,7 +813,7 @@ void owl_global_set_haveaim(owl_global *g)
   g->haveaim=1;
 }
 
-int owl_global_is_haveaim(owl_global *g)
+int owl_global_is_haveaim(const owl_global *g)
 {
   if (g->haveaim) return(1);
   return(0);
@@ -886,7 +829,7 @@ void owl_global_unset_ignore_aimlogin(owl_global *g)
     g->ignoreaimlogin = 0;
 }
 
-int owl_global_is_ignore_aimlogin(owl_global *g)
+int owl_global_is_ignore_aimlogin(const owl_global *g)
 {
     return g->ignoreaimlogin;
 }
@@ -896,7 +839,7 @@ void owl_global_set_havezephyr(owl_global *g)
   g->havezephyr=1;
 }
 
-int owl_global_is_havezephyr(owl_global *g)
+int owl_global_is_havezephyr(const owl_global *g)
 {
   if (g->havezephyr) return(1);
   return(0);
@@ -913,7 +856,9 @@ void owl_global_set_errsignal(owl_global *g, int signum, siginfo_t *siginfo)
   if (siginfo) {
     g->err_signal_info = *siginfo;
   } else {
-    memset(&(g->err_signal_info), 0, sizeof(siginfo_t));
+    siginfo_t si;
+    memset(&si, 0, sizeof(si));
+    g->err_signal_info = si;
   }
 }
 
@@ -934,19 +879,34 @@ owl_zbuddylist *owl_global_get_zephyr_buddylist(owl_global *g)
   return(&(g->zbuddies));
 }
 
+GList **owl_global_get_zaldlist(owl_global *g)
+{
+  return &(g->zaldlist);
+}
+
+int owl_global_get_pseudologin_notify(owl_global *g)
+{
+  return g->pseudologin_notify;
+}
+
+void owl_global_set_pseudologin_notify(owl_global *g, int notify)
+{
+  g->pseudologin_notify = notify;
+}
+
 struct termios *owl_global_get_startup_tio(owl_global *g)
 {
   return(&(g->startup_tio));
 }
 
-char * owl_global_intern(owl_global *g, char * string)
+owl_list *owl_global_get_io_dispatch_list(owl_global *g)
 {
-  return owl_obarray_insert(&(g->obarray), string);
+  return &(g->io_dispatch_list);
 }
 
-owl_list *owl_global_get_dispatchlist(owl_global *g)
+owl_list *owl_global_get_psa_list(owl_global *g)
 {
-  return &(g->dispatchlist);
+  return &(g->psa_list);
 }
 
 GList **owl_global_get_timerlist(owl_global *g)
@@ -954,7 +914,7 @@ GList **owl_global_get_timerlist(owl_global *g)
   return &(g->timerlist);
 }
 
-int owl_global_is_interrupted(owl_global *g) {
+int owl_global_is_interrupted(const owl_global *g) {
   return g->interrupted;
 }
 
@@ -965,3 +925,60 @@ void owl_global_set_interrupted(owl_global *g) {
 void owl_global_unset_interrupted(owl_global *g) {
   g->interrupted = 0;
 }
+
+void owl_global_setup_default_filters(owl_global *g)
+{
+  int i;
+  static const struct {
+    const char *name;
+    const char *desc;
+  } filters[] = {
+    { "personal",
+      "isprivate ^true$ and ( not type ^zephyr$ or ( class ^message  ) )" },
+    { "trash",
+      "class ^mail$ or opcode ^ping$ or type ^admin$ or ( not login ^none$ )" },
+    { "wordwrap", "not ( type ^admin$ or type ^zephyr$ )" },
+    { "ping", "opcode ^ping$" },
+    { "auto", "opcode ^auto$" },
+    { "login", "not login ^none$" },
+    { "reply-lockout", "class ^noc or class ^mail$" },
+    { "out", "direction ^out$" },
+    { "aim", "type ^aim$" },
+    { "zephyr", "type ^zephyr$" },
+    { "none", "false" },
+    { "all", "true" },
+    { NULL, NULL }
+  };
+
+  owl_function_debugmsg("startup: creating default filters");
+
+  for (i = 0; filters[i].name != NULL; i++)
+    owl_global_add_filter(g, owl_filter_new_fromstring(filters[i].name,
+                                                       filters[i].desc));
+}
+
+FILE *owl_global_get_debug_file_handle(owl_global *g) {
+  static char *open_file = NULL;
+  const char *filename = owl_global_get_debug_file(g);
+  if (g->debug_file == NULL ||
+      (open_file && strcmp(filename, open_file) != 0)) {
+    char *path;
+    int fd;
+
+    if (g->debug_file)
+      fclose(g->debug_file);
+
+    g->debug_file = NULL;
+
+    path = owl_sprintf("%s.%d", filename, getpid());
+    fd = open(path, O_CREAT|O_WRONLY|O_EXCL, 0600);
+    owl_free(path);
+
+    if (fd >= 0)
+      g->debug_file = fdopen(fd, "a");
+
+    owl_free(open_file);
+    open_file = owl_strdup(filename);
+  }
+  return g->debug_file;
+}
