@@ -1,4 +1,4 @@
-/*  Copyright (c) 2006-2010 The BarnOwl Developers. All rights reserved.
+/*  Copyright (c) 2006-2011 The BarnOwl Developers. All rights reserved.
  *  Copyright (c) 2004 James Kretchmar. All rights reserved.
  *
  *  This program is free software. You can redistribute it and/or
@@ -136,8 +136,8 @@ void owl_start_curses(void) {
 
   tcgetattr(0, &tio);
   tio.c_iflag &= ~(ISTRIP|IEXTEN);
-  tio.c_cc[VQUIT] = 0;
-  tio.c_cc[VSUSP] = 0;
+  tio.c_cc[VQUIT] = fpathconf(STDIN, _PC_VDISABLE);
+  tio.c_cc[VSUSP] = fpathconf(STDIN, _PC_VDISABLE);
   tcsetattr(0, TCSAFLUSH, &tio);
 
   /* screen init */
@@ -148,35 +148,10 @@ void owl_start_curses(void) {
   owl_start_color();
 }
 
-static void owl_setup_default_filters(void)
-{
-  int i;
-  static const struct {
-    const char *name;
-    const char *desc;
-  } filters[] = {
-    { "personal",
-      "isprivate ^true$ and ( not type ^zephyr$ or ( class ^message  ) )" },
-    { "trash",
-      "class ^mail$ or opcode ^ping$ or type ^admin$ or ( not login ^none$ )" },
-    { "wordwrap", "not ( type ^admin$ or type ^zephyr$ )" },
-    { "ping", "opcode ^ping$" },
-    { "auto", "opcode ^auto$" },
-    { "login", "not login ^none$" },
-    { "reply-lockout", "class ^noc or class ^mail$" },
-    { "out", "direction ^out$" },
-    { "aim", "type ^aim$" },
-    { "zephyr", "type ^zephyr$" },
-    { "none", "false" },
-    { "all", "true" },
-    { NULL, NULL }
-  };
-
-  owl_function_debugmsg("startup: creating default filters");
-
-  for (i = 0; filters[i].name != NULL; i++)
-    owl_global_add_filter(&g, owl_filter_new_fromstring(filters[i].name,
-                                                        filters[i].desc));
+void owl_shutdown_curses(void) {
+  endwin();
+  /* restore terminal settings */
+  tcsetattr(0, TCSAFLUSH, owl_global_get_startup_tio(&g));
 }
 
 /*
@@ -269,6 +244,8 @@ int owl_process_message(owl_message *m) {
   owl_perlconfig_newmsg(m, NULL);
   /* log the message if we need to */
   owl_log_message(m);
+  /* redraw the sepbar; TODO: don't violate layering */
+  owl_global_sepbar_dirty(&g);
 
   return 1;
 }
@@ -301,8 +278,6 @@ int owl_process_messages(owl_ps_action *d, void *p)
     /* redisplay if necessary */
     /* this should be optimized to not run if the new messages won't be displayed */
     owl_mainwin_redisplay(owl_global_get_mainwin(&g));
-    sepbar(NULL);
-    owl_global_set_needrefresh(&g);
   }
   return newmsgs;
 }
@@ -310,12 +285,9 @@ int owl_process_messages(owl_ps_action *d, void *p)
 void owl_process_input(const owl_io_dispatch *d, void *data)
 {
   owl_input j;
-  WINDOW *typwin;
-
-  typwin = owl_global_get_curs_typwin(&g);
 
   while (1) {
-    j.ch = wgetch(typwin);
+    j.ch = wgetch(g.input_pad);
     if (j.ch == ERR) return;
 
     j.uch = '\0';
@@ -338,7 +310,7 @@ void owl_process_input(const owl_io_dispatch *d, void *data)
       else bytes = 1;
       
       for (i = 1; i < bytes; i++) {
-        int tmp =  wgetch(typwin);
+        int tmp = wgetch(g.input_pad);
         /* If what we got was not a byte, or not a continuation byte */
         if (tmp > 0xff || !(tmp & 0x80 && ~tmp & 0x40)) {
           /* ill-formed UTF-8 code unit subsequence, put back the
@@ -388,6 +360,18 @@ void sigint_handler(int sig, siginfo_t *si, void *data)
   owl_global_set_interrupted(&g);
 }
 
+static int owl_errsignal_pre_select_action(owl_ps_action *a, void *data)
+{
+  siginfo_t si;
+  int signum;
+  if ((signum = owl_global_get_errsignal_and_clear(&g, &si)) > 0) {
+    owl_function_error("Got unexpected signal: %d %s  (code: %d band: %ld  errno: %d)",
+        signum, signum==SIGPIPE?"SIGPIPE":"SIG????",
+        si.si_code, si.si_band, si.si_errno);
+  }
+  return 0;
+}
+
 void owl_register_signal_handlers(void) {
   struct sigaction sigact;
 
@@ -457,36 +441,24 @@ void stderr_redirect_handler(const owl_io_dispatch *d, void *data)
 
 #endif /* OWL_STDERR_REDIR */
 
-void owl_zephyr_buddycheck_timer(owl_timer *t, void *data)
-{
-  if (owl_global_is_pseudologins(&g)) {
-    owl_function_debugmsg("Doing zephyr buddy check");
-    owl_function_zephyr_buddy_check(1);
-  }
-}
-
 static int owl_refresh_pre_select_action(owl_ps_action *a, void *data)
 {
+  owl_colorpair_mgr *cpmgr;
+
   /* if a resize has been scheduled, deal with it */
-  owl_global_resize(&g, 0, 0);
-
+  owl_global_check_resize(&g);
   /* update the terminal if we need to */
-  if (owl_global_is_needrefresh(&g)) {
-    /* these are here in case a resize changes the windows */
-    WINDOW *sepwin = owl_global_get_curs_sepwin(&g);
-    WINDOW *typwin = owl_global_get_curs_typwin(&g);
-
-    /* push all changed windows to screen */
-    update_panels();
-    /* leave the cursor in the appropriate window */
-    if (!owl_popwin_is_active(owl_global_get_popwin(&g))
-	&& owl_global_get_typwin(&g)) {
-      owl_function_set_cursor(typwin);
-    } else {
-      owl_function_set_cursor(sepwin);
-    }
-    doupdate();
-    owl_global_set_noneedrefresh(&g);
+  owl_window_redraw_scheduled();
+  /* On colorpair shortage, reset and redraw /everything/. NOTE: if
+   * the current screen uses too many colorpairs, this draws
+   * everything twice. But this is unlikely; COLOR_PAIRS is 64 with
+   * 8+1 colors, and 256^2 with 256+1 colors. (+1 for default.) */
+  cpmgr = owl_global_get_colorpair_mgr(&g);
+  if (cpmgr->overflow) {
+    owl_function_debugmsg("colorpairs: color shortage; reset pairs and redraw. COLOR_PAIRS = %d", COLOR_PAIRS);
+    owl_fmtext_reset_colorpairs(cpmgr);
+    owl_function_full_redisplay();
+    owl_window_redraw_scheduled();
   }
   return 0;
 }
@@ -514,8 +486,6 @@ int main(int argc, char **argv, char **env)
   owl_parse_options(argc, argv, &opts);
   g.load_initial_subs = opts.load_initial_subs;
 
-  owl_function_debugmsg("startup: Finished parsing arguments");
-
   owl_register_signal_handlers();
   owl_start_curses();
 
@@ -557,7 +527,7 @@ int main(int argc, char **argv, char **env)
   owl_function_debugmsg("startup: processing config file");
 
   owl_global_pop_context(&g);
-  owl_global_push_context(&g, OWL_CTX_READCONFIG, NULL, NULL);
+  owl_global_push_context(&g, OWL_CTX_READCONFIG, NULL, NULL, NULL);
 
   perlerr=owl_perlconfig_initperl(opts.configfile, &argc, &argv, &env);
   if (perlerr) {
@@ -571,7 +541,7 @@ int main(int argc, char **argv, char **env)
 
   owl_global_complete_setup(&g);
 
-  owl_setup_default_filters();
+  owl_global_setup_default_filters(&g);
 
   /* set the current view */
   owl_function_debugmsg("startup: setting the current view");
@@ -592,8 +562,9 @@ int main(int argc, char **argv, char **env)
   owl_function_debugmsg("startup: creating splash message");
   owl_function_adminmsg("",
     "-----------------------------------------------------------------------\n"
-    "Welcome to barnowl version " OWL_VERSION_STRING ".  Press 'h' for on-line help.\n"
+    "Welcome to barnowl version " OWL_VERSION_STRING ".\n"
     "To see a quick introduction, type ':show quickstart'.                  \n"
+    "Press 'h' for on-line help.                                            \n"
     "                                                                       \n"
     "BarnOwl is free software. Type ':show license' for more                \n"
     "information.                                                     ^ ^   \n"
@@ -601,22 +572,11 @@ int main(int argc, char **argv, char **env)
     "Please report any bugs or suggestions to bug-barnowl@mit.edu    (   )  \n"
     "-----------------------------------------------------------------m-m---\n"
   );
-  sepbar(NULL);
 
   /* process the startup file */
   owl_function_debugmsg("startup: processing startup file");
   owl_function_source(NULL);
 
-  update_panels();
-
-  /* Set the default style */
-  owl_function_debugmsg("startup: setting startup and default style");
-  if (0 != strcmp(owl_global_get_default_style(&g), "__unspecified__")) {
-    /* the style was set by the user: leave it alone */
-  } else {
-    owl_global_set_default_style(&g, "default");
-  }
-
   owl_function_debugmsg("startup: set style for the view: %s", owl_global_get_default_style(&g));
   s = owl_global_get_style_by_name(&g, owl_global_get_default_style(&g));
   if(s)
@@ -627,32 +587,17 @@ int main(int argc, char **argv, char **env)
   owl_function_debugmsg("startup: setting context interactive");
 
   owl_global_pop_context(&g);
-  owl_global_push_context(&g, OWL_CTX_READCONFIG|OWL_CTX_RECV, NULL, "recv");
-
-  owl_select_add_timer(180, 180, owl_zephyr_buddycheck_timer, NULL, NULL);
-
-  /* If we ever deprecate the mainloop hook, remove this. */
-  owl_select_add_timer(0, 1, owl_perlconfig_mainloop, NULL, NULL);
+  owl_global_push_context(&g, OWL_CTX_INTERACTIVE|OWL_CTX_RECV, NULL, "recv", NULL);
 
   owl_select_add_pre_select_action(owl_refresh_pre_select_action, NULL, NULL);
   owl_select_add_pre_select_action(owl_process_messages, NULL, NULL);
+  owl_select_add_pre_select_action(owl_errsignal_pre_select_action, NULL, NULL);
 
   owl_function_debugmsg("startup: entering main loop");
-  /* main loop */
-  while (1) {
-    /* select on FDs we know about. */
-    owl_select();
+  owl_select_run_loop();
 
-    /* Log any error signals */
-    {
-      siginfo_t si;
-      int signum;
-      if ((signum = owl_global_get_errsignal_and_clear(&g, &si)) > 0) {
-	owl_function_error("Got unexpected signal: %d %s  (code: %d band: %ld  errno: %d)",
-			   signum, signum==SIGPIPE?"SIGPIPE":"SIG????",
-			   si.si_code, si.si_band, si.si_errno);
-      }
-    }
-
-  }
+  /* Shut down everything. */
+  owl_zephyr_shutdown();
+  owl_shutdown_curses();
+  return 0;
 }
