@@ -3,7 +3,7 @@ use warnings;
 
 package BarnOwl::Message::Zephyr;
 
-use constant WEBZEPHYR_PRINCIPAL => "daemon.webzephyr";
+use constant WEBZEPHYR_PRINCIPAL => "daemon/webzephyr.mit.edu";
 use constant WEBZEPHYR_CLASS     => "webzephyr";
 use constant WEBZEPHYR_OPCODE    => "webzephyr";
 
@@ -12,10 +12,16 @@ use base qw( BarnOwl::Message );
 sub strip_realm {
     my $sender = shift;
     my $realm = BarnOwl::zephyr_getrealm();
-    $sender =~ s/\@$realm$//;
+    $sender =~ s/\@\Q$realm\E$//;
     return $sender;
 }
 
+sub principal_realm {
+    my $principal = shift;
+    my ($user, $realm) = split(/@/,$principal);
+    return $realm;
+}
+
 sub login_type {
     return (shift->zsig eq "") ? "(PSEUDO)" : "";
 }
@@ -82,24 +88,20 @@ sub pretty_recipient {
 sub context_reply_cmd {
     my $mclass = shift;
     my $minstance = shift;
-    my $class = "";
+    my @class;
     if (lc($mclass) ne "message") {
-        $class = "-c " . BarnOwl::quote($mclass);
+        @class = ('-c', $mclass);
     }
-    my $instance = "";
+    my @instance;
     if (lc($minstance) ne "personal") {
-        $instance = "-i " . BarnOwl::quote($minstance);
-    }
-    if (($class eq "") or  ($instance eq "")) {
-        return $class . $instance;
-    } else {
-        return $class . " " . $instance;
+        @instance = ('-i', $minstance);
     }
+    return (@class, @instance);
 }
 
 sub personal_context {
     my ($m) = @_;
-    return context_reply_cmd($m->class, $m->instance);
+    return BarnOwl::quote(context_reply_cmd($m->class, $m->instance));
 }
 
 sub short_personal_context {
@@ -177,33 +179,47 @@ sub replycmd {
         }
     }
 
-    my $cmd;
+    my @cmd;
     if(lc $self->opcode eq 'crypt' and ( not $sender or $self->is_private)) {
         # Responses to zcrypted messages should be zcrypted, so long as we
         # aren't switching to personals
-        $cmd = 'zcrypt';
+        @cmd = ('zcrypt');
     } else {
-        $cmd = 'zwrite';
+        @cmd = ('zwrite');
     }
 
-    my $context_part = context_reply_cmd($class, $instance);
-    $cmd .= " " . $context_part unless ($context_part eq '');
+    push @cmd, context_reply_cmd($class, $instance);
+
     if ($to ne '') {
         $to = strip_realm($to);
-        if (defined $cc) {
+        if (defined $cc and not $sender) {
             my @cc = grep /^[^-]/, ($to, split /\s+/, $cc);
             my %cc = map {$_ => 1} @cc;
+            # this isn't quite right - it doesn't strip off the
+            # user if the message was addressed to them by fully qualified
+            # name
             delete $cc{strip_realm(BarnOwl::zephyr_getsender())};
             @cc = keys %cc;
-            $cmd .= " -C " . join(" ", @cc);
+
+            my $sender_realm = principal_realm($self->sender);
+            if (BarnOwl::zephyr_getrealm() ne $sender_realm) {
+                @cc = map {
+                    if($_ !~ /@/) {
+                       "${_}\@${sender_realm}";
+                    } else {
+                        $_;
+                    }
+                } @cc;
+            }
+            push @cmd, '-C', @cc;
         } else {
             if(BarnOwl::getvar('smartstrip') eq 'on') {
                 $to = BarnOwl::zephyr_smartstrip_user($to);
             }
-            $cmd .= " $to";
+            push @cmd, $to;
         }
     }
-    return $cmd;
+    return BarnOwl::quote(@cmd);
 }
 
 sub replysendercmd {
