@@ -11,7 +11,7 @@ extern XS(boot_BarnOwl);
 extern XS(boot_DynaLoader);
 /* extern XS(boot_DBI); */
 
-static void owl_perl_xs_init(pTHX)
+void owl_perl_xs_init(pTHX) /* noproto */
 {
   const char *file = __FILE__;
   dXSUB_SYS;
@@ -185,7 +185,7 @@ owl_message * owl_perlconfig_hashref2message(SV *msg)
 {
   owl_message * m;
   HE * ent;
-  I32 count, len;
+  I32 len;
   const char *key,*val;
   HV * hash;
   struct tm tm;
@@ -195,7 +195,7 @@ owl_message * owl_perlconfig_hashref2message(SV *msg)
   m = owl_malloc(sizeof(owl_message));
   owl_message_init(m);
 
-  count = hv_iterinit(hash);
+  hv_iterinit(hash);
   while((ent = hv_iternext(hash))) {
     key = hv_iterkey(ent, &len);
     val = SvPV_nolen(hv_iterval(hash, ent));
@@ -362,6 +362,8 @@ char *owl_perlconfig_initperl(const char * file, int *Pargc, char ***Pargv, char
   owl_global_set_perlinterp(&g, p);
   perl_construct(p);
 
+  PL_exit_flags |= PERL_EXIT_DESTRUCT_END;
+
   owl_global_set_no_have_config(&g);
 
   ret=perl_parse(p, owl_perl_xs_init, 2, (char **)args, NULL);
@@ -486,7 +488,7 @@ void owl_perlconfig_new_command(const char *name)
   XPUSHs(sv_2mortal(owl_new_sv(name)));
   PUTBACK;
 
-  call_pv("BarnOwl::Hooks::_new_command", G_SCALAR|G_VOID);
+  call_pv("BarnOwl::Hooks::_new_command", G_SCALAR|G_VOID|G_EVAL);
 
   SPAGAIN;
 
@@ -581,19 +583,6 @@ void owl_perlconfig_dec_refcnt(void *data)
   SvREFCNT_dec(v);
 }
 
-void owl_perlconfig_mainloop(owl_timer *t, void *data)
-{
-  dSP;
-  if (!owl_perlconfig_is_function("BarnOwl::Hooks::_mainloop_hook"))
-    return;
-  PUSHMARK(SP) ;
-  call_pv("BarnOwl::Hooks::_mainloop_hook", G_DISCARD|G_EVAL);
-  if(SvTRUE(ERRSV)) {
-    owl_function_error("%s", SvPV_nolen(ERRSV));
-  }
-  return;
-}
-
 void owl_perlconfig_io_dispatch(const owl_io_dispatch *d, void *data)
 {
   SV *cb = data;
