@@ -2,6 +2,7 @@
 
 static int dispatch_active = 0;
 static int psa_active = 0;
+static int loop_active = 0;
 
 int _owl_select_timer_cmp(const owl_timer *t1, const owl_timer *t2) {
   return t1->time - t2->time;
@@ -11,7 +12,7 @@ int _owl_select_timer_eq(const owl_timer *t1, const owl_timer *t2) {
   return t1 == t2;
 }
 
-owl_timer *owl_select_add_timer(int after, int interval, void (*cb)(owl_timer *, void *), void (*destroy)(owl_timer*), void *data)
+owl_timer *owl_select_add_timer(const char* name, int after, int interval, void (*cb)(owl_timer *, void *), void (*destroy)(owl_timer*), void *data)
 {
   owl_timer *t = owl_malloc(sizeof(owl_timer));
   GList **timers = owl_global_get_timerlist(&g);
@@ -21,6 +22,7 @@ owl_timer *owl_select_add_timer(int after, int interval, void (*cb)(owl_timer *,
   t->callback = cb;
   t->destroy = destroy;
   t->data = data;
+  t->name = name ? owl_strdup(name) : NULL;
 
   *timers = g_list_insert_sorted(*timers, t,
                                  (GCompareFunc)_owl_select_timer_cmp);
@@ -35,6 +37,7 @@ void owl_select_remove_timer(owl_timer *t)
     if(t->destroy) {
       t->destroy(t);
     }
+    owl_free(t->name);
     owl_free(t);
   }
 }
@@ -241,7 +244,7 @@ int owl_select_aim_hack(fd_set *rfds, fd_set *wfds)
 
   max_fd = 0;
   sess = owl_global_get_aimsess(&g);
-  for (cur = sess->connlist, max_fd = 0; cur; cur = cur->next) {
+  for (cur = sess->connlist; cur; cur = cur->next) {
     if (cur->fd != -1) {
       FD_SET(cur->fd, rfds);
       if (cur->status & AIM_CONN_STATUS_INPROGRESS) {
@@ -272,6 +275,11 @@ void owl_select_mask_signals(sigset_t *oldmask) {
   sigset_t set;
 
   sigemptyset(&set);
+  sigaddset(&set, SIGWINCH);
+  sigaddset(&set, SIGALRM);
+  sigaddset(&set, SIGPIPE);
+  sigaddset(&set, SIGTERM);
+  sigaddset(&set, SIGHUP);
   sigaddset(&set, SIGINT);
   sigprocmask(SIG_BLOCK, &set, oldmask);
 }
@@ -430,3 +438,16 @@ void owl_select(void)
     owl_select_io_dispatch(&r, &w, &e, max_fd);
   }
 }
+
+void owl_select_run_loop(void)
+{
+  loop_active = 1;
+  while (loop_active) {
+    owl_select();
+  }
+}
+
+void owl_select_quit_loop(void)
+{
+  loop_active = 0;
+}
