@@ -109,7 +109,7 @@ static owl_variable variables_to_init[] = {
 	       "the name of the class to which the message was sent.\n" ),
 
   OWLVAR_ENUM( "loggingdirection" /* %OwlVarStub */, OWL_LOGGING_DIRECTION_BOTH,
-	       "specifices which kind of messages should be logged",
+	       "specifies which kind of messages should be logged",
 	       "Can be one of 'both', 'in', or 'out'.  If 'in' is\n"
 	       "selected, only incoming messages are logged, if 'out'\n"
 	       "is selected only outgoing messages are logged.  If 'both'\n"
@@ -137,7 +137,7 @@ static owl_variable variables_to_init[] = {
 		    "When this is enabled, Owl will periodically check the zephyr\n"
 		    "location of users in your .anyone file.  If a user is present\n"
 		    "but sent no login message, or a user is not present that sent no\n"
-		    "logout message, a pseudo login or logout message wil be created\n",
+		    "logout message, a pseudo login or logout message will be created\n",
 		    NULL, owl_variable_pseudologins_set, NULL),
 
   OWLVAR_BOOL( "ignorelogins" /* %OwlVarStub */, 0,
@@ -150,7 +150,7 @@ static owl_variable variables_to_init[] = {
 		 "name of a filter controlling which messages to log",
 
 		 "If non empty, any messages matching the given filter will be logged.\n"
-		 "This is a completely separate mechanisim from the other logging\n"
+		 "This is a completely separate mechanism from the other logging\n"
 		 "variables like logging, classlogging, loglogins, loggingdirection,\n"
 		 "etc.  If you want this variable to control all logging, make sure\n"
 		 "all other logging variables are in their default state.\n"),
@@ -198,7 +198,7 @@ static owl_variable variables_to_init[] = {
 
   OWLVAR_PATH( "newmsgproc" /* %OwlVarStub:newmsgproc */, NULL,
 	       "name of a program to run when new messages are present",
-	       "The named program will be run when owl recevies new.\n"
+	       "The named program will be run when owl receives new\n"
 	       "messages.  It will not be run again until the first\n"
 	       "instance exits"),
 
@@ -213,7 +213,8 @@ static owl_variable variables_to_init[] = {
 		 "zsig perl function",
 		 "Called every time you start a zephyrgram without an\n"
 		 "explicit zsig.  The default setting implements the policy\n"
-		 "descripted in the documentation for the 'zsig' variable.\n"),
+		 "described in the documentation for the 'zsig' variable.\n"
+		 "See also BarnOwl::random_zephyr_signature().\n"),
 
   OWLVAR_STRING( "zsig" /* %OwlVarStub */, "",
 	         "zephyr signature",
@@ -267,7 +268,7 @@ static owl_variable variables_to_init[] = {
   OWLVAR_STRING_FULL( "tty" /* %OwlVarStub */, "", "tty name for zephyr location", "",
 		      NULL, owl_variable_tty_set, NULL),
 
-  OWLVAR_STRING( "default_style" /* %OwlVarStub */, "__unspecified__",
+  OWLVAR_STRING( "default_style" /* %OwlVarStub */, "default",
 		 "name of the default formatting style",
 		 "This sets the default message formatting style.\n"
 		 "Styles may be created with the 'style' command.\n"
@@ -280,29 +281,24 @@ static owl_variable variables_to_init[] = {
 
 
   OWLVAR_INT(    "edit:maxfillcols" /* %OwlVarStub:edit_maxfillcols */, 70,
-	         "maximum number of columns for M-q to fill text to",
-		 "This specifies the maximum number of columns for M-q\n"
-		 "to fill text to.  If set to 0, ther will be no maximum\n"
-		 "limit.  In all cases, the current width of the screen\n"
-		 "will also be taken into account.  It will be used instead\n"
-		 "if it is narrower than the maximum, or if this\n"
-		 "is set to 0.\n" ),
+	         "maximum number of columns for M-q (edit:fill-paragraph) to fill text to",
+                 "This specifies the maximum number of columns for M-q to fill text\n"
+                 "to.  If set to 0, M-q will wrap to the width of the window, and\n"
+                 "values less than 0 disable M-q entirely.\n"),
 
   OWLVAR_INT(    "edit:maxwrapcols" /* %OwlVarStub:edit_maxwrapcols */, 70,
 	         "maximum number of columns for line-wrapping",
-		 "This specifies the maximum number of columns for\n"
-		 "auto-line-wrapping.  If set to 0, ther will be no maximum\n"
-		 "limit.  In all cases, the current width of the screen\n"
-		 "will also be taken into account.  It will be used instead\n"
-		 "if it is narrower than the maximum, or if this\n"
-		 "is set to 0.\n\n"
-		 "It is recommended that outgoing messages be no wider\n"
-		 "than 70 columns, as a courtesy to recipients.\n"),
+                 "This specifies the maximum number of columns for\n"
+                 "auto-line-wrapping.  If set to 0, text will be wrapped at the\n"
+                 "window width. Values less than 0 disable automatic wrapping.\n"
+                 "\n"
+                 "As a courtesy to recipients, it is recommended that outgoing\n"
+                 "Zephyr messages be no wider than 70 columns.\n"),
 
   OWLVAR_INT( "aim_ignorelogin_timer" /* %OwlVarStub */, 15,
 	      "number of seconds after AIM login to ignore login messages",
 	      "This specifies the number of seconds to wait after an\n"
-	      "AIM login before allowing the recipt of AIM login notifications.\n"
+	      "AIM login before allowing the receipt of AIM login notifications.\n"
 	      "By default this is set to 15.  If you would like to view login\n"
 	      "notifications of buddies as soon as you login, set it to 0 instead."),
 
@@ -405,7 +401,7 @@ int owl_variable_typewinsize_set(owl_variable *v, const void *newval)
 {
   int rv;
   rv = owl_variable_int_set_default(v, newval);
-  if (0 == rv) owl_function_resize();
+  if (0 == rv) owl_mainpanel_layout_contents(&g.mainpanel);
   return(rv);
 }
 
@@ -433,9 +429,19 @@ int owl_variable_aaway_set(owl_variable *v, const void *newval)
 
 int owl_variable_pseudologins_set(owl_variable *v, const void *newval)
 {
+  static owl_timer *timer = NULL;
   if (newval) {
     if (*(const int*)newval == 1) {
       owl_function_zephyr_buddy_check(0);
+      if (timer == NULL) {
+        timer = owl_select_add_timer("owl_zephyr_buddycheck_timer",
+                                     180, 180, owl_zephyr_buddycheck_timer, NULL, NULL);
+      }
+    } else {
+      if (timer != NULL) {
+        owl_select_remove_timer(timer);
+        timer = NULL;
+      }
     }
   }
   return owl_variable_bool_set_default(v, newval);
@@ -474,6 +480,10 @@ int owl_variable_dict_setup(owl_vardict *vd) {
   for (var = variables_to_init; var->name != NULL; var++) {
     cur = owl_malloc(sizeof(owl_variable));
     *cur = *var;
+    /* strdup all the strings so we can delete them consistently. */
+    cur->name = owl_strdup(var->name);
+    cur->summary = owl_strdup(var->summary);
+    cur->description = owl_strdup(var->description);
     switch (cur->type) {
     case OWL_VARIABLE_OTHER:
       cur->set_fn(cur, cur->pval_default);
@@ -635,6 +645,9 @@ void owl_variable_dict_namelist_cleanup(owl_list *l)
 void owl_variable_delete(owl_variable *v)
 {
   if (v->delete_fn) v->delete_fn(v);
+  owl_free(v->name);
+  owl_free(v->summary);
+  owl_free(v->description);
   owl_free(v);
 }
 
