@@ -147,8 +147,6 @@ void owl_zephyr_load_initial_subs(void) {
 
   if (ret_sd || ret_bd || ret_u) {
     owl_function_error("Error loading zephyr subscriptions");
-  } else if (ret_u!=-1) {
-    owl_global_add_userclue(&g, OWL_USERCLUE_CLASSES);
   }
 
   /* load login subscriptions */
@@ -178,8 +176,15 @@ int owl_zephyr_shutdown(void)
 int owl_zephyr_zpending(void)
 {
 #ifdef HAVE_LIBZEPHYR
-  if(owl_global_is_havezephyr(&g))
-    return(ZPending());
+  Code_t code;
+  if(owl_global_is_havezephyr(&g)) {
+    if((code = ZPending()) < 0) {
+      owl_function_debugmsg("Error (%s) in ZPending()\n",
+                            error_message(code));
+      return 0;
+    }
+    return code;
+  }
 #endif
   return 0;
 }
@@ -257,6 +262,7 @@ int owl_zephyr_loadsubs(const char *filename, int error_on_nofile)
   subsfile = owl_zephyr_dotfile(".zephyr.subs", filename);
 
   if (stat(subsfile, &statbuff) != 0) {
+    owl_free(subsfile);
     if (error_on_nofile == 1)
       return -1;
     return 0;
@@ -327,7 +333,6 @@ int owl_zephyr_loadbarnowldefaultsubs(void)
   int count, ret;
 
   subs = owl_malloc(sizeof(ZSubscription_t) * subSize);
-  ret = 0;
   ZResetAuthentication();
   count=0;
 
@@ -346,11 +351,13 @@ int owl_zephyr_loadbarnowldefaultsubs(void)
 int owl_zephyr_loaddefaultsubs(void)
 {
 #ifdef HAVE_LIBZEPHYR
-  ZSubscription_t subs[10];
+  if (owl_global_is_havezephyr(&g)) {
+    ZSubscription_t subs[10];
     
-  if (ZSubscribeTo(subs,0,0) != ZERR_NONE) {
-    owl_function_error("Error subscribing to default zephyr notifications.");
-    return(-1);
+    if (ZSubscribeTo(subs,0,0) != ZERR_NONE) {
+      owl_function_error("Error subscribing to default zephyr notifications.");
+      return(-1);
+    }
   }
   return(0);
 #else
@@ -372,8 +379,11 @@ int owl_zephyr_loadloginsubs(const char *filename)
   subs = owl_malloc(numSubs * sizeof(ZSubscription_t));
   subsfile = owl_zephyr_dotfile(".anyone", filename);
 
-  if (stat(subsfile, &statbuff) == -1)
+  if (stat(subsfile, &statbuff) == -1) {
+    owl_free(subs);
+    owl_free(subsfile);
     return 0;
+  }
 
   ZResetAuthentication();
   count = 0;
@@ -848,6 +858,7 @@ void owl_zephyr_zaway(const owl_message *m)
 #ifdef HAVE_LIBZEPHYR
   char *tmpbuff, *myuser, *to;
   owl_message *mout;
+  owl_zwrite *z;
   
   /* bail if it doesn't look like a message we should reply to.  Some
    * of this defined by the way zaway(1) works
@@ -883,10 +894,15 @@ void owl_zephyr_zaway(const owl_message *m)
   owl_free(myuser);
   owl_free(to);
 
+  z = owl_zwrite_new(tmpbuff);
+  owl_zwrite_set_message(z, owl_global_get_zaway_msg(&g));
+  owl_zwrite_set_zsig(z, "Automated reply:");
+
   /* display the message as an admin message in the receive window */
-  mout=owl_function_make_outgoing_zephyr(owl_global_get_zaway_msg(&g), tmpbuff, "Automated reply:");
+  mout=owl_function_make_outgoing_zephyr(z);
   owl_global_messagequeue_addmsg(&g, mout);
   owl_free(tmpbuff);
+  owl_zwrite_delete(z);
 #endif
 }
 
@@ -935,6 +951,7 @@ char *owl_zephyr_zlocate(const char *user, int auth)
       result = p;
     }
   }
+  owl_free(myuser);
 
   return result;
 #else
@@ -994,7 +1011,7 @@ void owl_zephyr_delsub(const char *filename, const char *class, const char *inst
   linesdeleted = owl_util_file_deleteline(subsfile, line, 1);
   if (linesdeleted > 0) {
     owl_function_makemsg("Subscription removed");
-  } else {
+  } else if (linesdeleted == 0) {
     owl_function_error("No subscription present in %s", subsfile);
   }
   owl_free(subsfile);
@@ -1211,37 +1228,42 @@ char *long_zuser(const char *in)
 }
 
 /* strip out the instance from a zsender's principal.  Preserves the
- * realm if present.  daemon.webzephyr is a special case.  The
- * caller must free the return
+ * realm if present.  Leave host/ and daemon/ krb5 principals
+ * alone. Also leave rcmd. and daemon. krb4 principals alone. The
+ * caller must free the return.
  */
 char *owl_zephyr_smartstripped_user(const char *in)
 {
-  char *ptr, *realm, *out;
+  char *slash, *dot, *realm, *out;
 
-  out=owl_strdup(in);
+  out = owl_strdup(in);
 
   /* bail immeaditly if we don't have to do any work */
-  ptr=strchr(out, '.');
-  if (!strchr(out, '/') && !ptr) {
-    /* no '/' and no '.' */
+  slash = strchr(out, '/');
+  dot = strchr(out, '.');
+  if (!slash && !dot) {
     return(out);
   }
-  if (ptr && strchr(out, '@') && (ptr > strchr(out, '@'))) {
-    /* There's a '.' but it's in the realm */
+
+  if (!strncasecmp(out, OWL_ZEPHYR_NOSTRIP_HOST, strlen(OWL_ZEPHYR_NOSTRIP_HOST)) ||
+      !strncasecmp(out, OWL_ZEPHYR_NOSTRIP_RCMD, strlen(OWL_ZEPHYR_NOSTRIP_RCMD)) ||
+      !strncasecmp(out, OWL_ZEPHYR_NOSTRIP_DAEMON5, strlen(OWL_ZEPHYR_NOSTRIP_DAEMON5)) ||
+      !strncasecmp(out, OWL_ZEPHYR_NOSTRIP_DAEMON4, strlen(OWL_ZEPHYR_NOSTRIP_DAEMON4))) {
     return(out);
   }
-  if (!strncasecmp(out, OWL_WEBZEPHYR_PRINCIPAL, strlen(OWL_WEBZEPHYR_PRINCIPAL))) {
+
+  realm = strchr(out, '@');
+  if (!slash && dot && realm && (dot > realm)) {
+    /* There's no '/', and the first '.' is in the realm */
     return(out);
   }
 
   /* remove the realm from out, but hold on to it */
-  realm=strchr(out, '@');
   if (realm) realm[0]='\0';
 
   /* strip */
-  ptr=strchr(out, '.');
-  if (!ptr) ptr=strchr(out, '/');
-  ptr[0]='\0';
+  if (slash) slash[0] = '\0';  /* krb5 style user/instance */
+  else if (dot) dot[0] = '\0'; /* krb4 style user.instance */
 
   /* reattach the realm if we had one */
   if (realm) {
@@ -1363,6 +1385,16 @@ void owl_zephyr_process_pseudologin(void *n)
 }
 #endif
 
+void owl_zephyr_buddycheck_timer(owl_timer *t, void *data)
+{
+  if (owl_global_is_pseudologins(&g)) {
+    owl_function_debugmsg("Doing zephyr buddy check");
+    owl_function_zephyr_buddy_check(1);
+  } else {
+    owl_function_debugmsg("Warning: owl_zephyr_buddycheck_timer call pointless; timer should have been disabled");
+  }
+}
+
 /*
  * Process zephyrgrams from libzephyr's queue. To prevent starvation,
  * process a maximum of OWL_MAX_ZEPHYRGRAMS_TO_PROCESS.
@@ -1377,11 +1409,16 @@ static int _owl_zephyr_process_events(void)
   int zpendcount=0;
 #ifdef HAVE_LIBZEPHYR
   ZNotice_t notice;
+  Code_t code;
   owl_message *m=NULL;
 
   while(owl_zephyr_zpending() && zpendcount < OWL_MAX_ZEPHYRGRAMS_TO_PROCESS) {
     if (owl_zephyr_zpending()) {
-      ZReceiveNotice(&notice, NULL);
+      if ((code = ZReceiveNotice(&notice, NULL)) != ZERR_NONE) {
+        owl_function_debugmsg("Error: %s while calling ZReceiveNotice\n",
+                              error_message(code));
+        continue;
+      }
       zpendcount++;
 
       /* is this an ack from a zephyr we sent? */
